@@ -1,5 +1,5 @@
 /*
- *  $Id: asapinstance.h,v 1.15 2004/12/09 15:29:05 dreibh Exp $
+ *  $Id: asapinstance.h 748 2005-09-13 08:40:04Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -42,9 +42,8 @@
 #include "dispatcher.h"
 #include "tagitem.h"
 #include "rserpoolmessage.h"
-#include "messagebuffer.h"
 #include "poolhandlespacemanagement.h"
-#include "servertable.h"
+#include "registrartable.h"
 
 
 #ifdef __cplusplus
@@ -55,40 +54,37 @@ extern "C" {
 struct ASAPInstance
 {
    struct Dispatcher*                         StateMachine;
-   unsigned long long                         RegistrarConnectionTimeStamp;
-   int                                        RegistrarSocket;
-   RegistrarIdentifierType                    RegistrarID;
-   int                                        RegistrarSocketProtocol;
 
-   struct ServerTable*                        RegistrarTable;
+   int                                        RegistrarHuntSocket;
+   int                                        RegistrarSocket;
+   RegistrarIdentifierType                    RegistrarIdentifier;
+   unsigned long long                         RegistrarConnectionTimeStamp;
+
+   struct RegistrarTable*                     RegistrarSet;
    struct ST_CLASS(PoolHandlespaceManagement) Cache;
    struct ST_CLASS(PoolHandlespaceManagement) OwnPoolElements;
 
-   char*                                      AsapServerAnnounceConfigFile;
-   char*                                      AsapRegistrarsConfigFile;
-
    unsigned long long                         CacheElementTimeout;
-   unsigned long long                         CacheMaintenanceInterval;
 
+   struct FDCallback                          RegistrarHuntFDCallback;
    struct FDCallback                          RegistrarFDCallback;
+
    size_t                                     RegistrarRequestMaxTrials;
    unsigned long long                         RegistrarRequestTimeout;
    unsigned long long                         RegistrarResponseTimeout;
-
-   struct MessageBuffer*                      Buffer;
 };
 
 
-#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT                5000000
-#define ASAP_DEFAULT_NAMESERVER_ANNOUNCE_ADDRESS "239.0.0.1:3863"
-#define ASAP_DEFAULT_NAMESERVER_ANNOUNCE_TIMEOUT         30000000
-#define ASAP_DEFAULT_NAMESERVER_CONNECT_MAXTRIALS               3
-#define ASAP_DEFAULT_NAMESERVER_CONNECT_TIMEOUT           1500000
-#define ASAP_DEFAULT_NAMESERVER_REQUEST_MAXTRIALS               1
-#define ASAP_DEFAULT_NAMESERVER_REQUEST_TIMEOUT           1500000
-#define ASAP_DEFAULT_NAMESERVER_RESPONSE_TIMEOUT          1500000
+#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT               5000000
+#define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS "239.0.0.1:3863"
+#define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_TIMEOUT          5000000
+#define ASAP_DEFAULT_REGISTRAR_CONNECT_MAXTRIALS               3
+#define ASAP_DEFAULT_REGISTRAR_CONNECT_TIMEOUT           1500000
+#define ASAP_DEFAULT_REGISTRAR_REQUEST_MAXTRIALS               1
+#define ASAP_DEFAULT_REGISTRAR_REQUEST_TIMEOUT           1500000
+#define ASAP_DEFAULT_REGISTRAR_RESPONSE_TIMEOUT          1500000
 
-#define ASAP_BUFFER_SIZE                                    65536
+#define ASAP_BUFFER_SIZE                                   65536
 
 
 /**
@@ -156,7 +152,7 @@ unsigned int asapInstanceReportFailure(struct ASAPInstance*            asapInsta
   * @param poolElementNodes Reference to variable containing maximum amount of pool element nodes to obtain. After function call, this variable contains actual amount of pool element nodes obtained.
   * @return RSPERR_OKAY in case of success; error code otherwise.
   */
-unsigned int asapInstanceNameResolution(
+unsigned int asapInstanceHandleResolution(
                 struct ASAPInstance*               asapInstanceInstance,
                 struct PoolHandle*                 poolHandle,
                 struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
