@@ -1,5 +1,5 @@
 /*
- *  $Id: examplepe.c,v 1.21 2004/12/16 16:16:58 dreibh Exp $
+ *  $Id: examplepe.c 735 2005-08-26 19:32:12Z tuexen $
  *
  * RSerPool implementation.
  *
@@ -134,9 +134,9 @@ static void handleServiceRequest(GList**        clientList,
       if(tags[0].Data == 0) {
          client->IsNewClient = false;
          buffer[received]  = 0x00;
-         printf("Echo %u %u> %s", client->CounterStart,
-                                  client->Counter,
-                                  buffer);
+         printf("Echo %u %u> %s\n", client->CounterStart,
+                                    client->Counter,
+                                    buffer);
          client->Counter++;
          rspSessionWrite(client->Session, (char*)&buffer, received, NULL);
 
@@ -148,10 +148,12 @@ static void handleServiceRequest(GList**        clientList,
          handleCookieEcho(client, (void*)&buffer, received);
       }
    }
+   /*
    else if(received == RspRead_Timeout) {
       puts("INTERNAL ERROR: Timeout occurred! This should never happen!");
       exit(1);
    }
+   */
    else {
       *clientList = g_list_remove(*clientList, (gpointer)client);
       rspDeleteSession(client->Session, NULL);
@@ -170,7 +172,7 @@ static void* rsplibMainLoop(void* args)
    struct timeval timeout;
    while(!RsplibThreadStop) {
       timeout.tv_sec  = 0;
-      timeout.tv_usec = 50000;
+      timeout.tv_usec = 250000;
       rspSelect(0, NULL, NULL, NULL, &timeout);
    }
    return(NULL);
@@ -180,7 +182,7 @@ static void* rsplibMainLoop(void* args)
 /* ###### Main program ################################################### */
 int main(int argc, char** argv)
 {
-   uint32_t                      identifier        = 0;
+   uint32_t                      identifier = 0;
    struct PoolElementDescriptor* poolElement;
    struct TagItem                tags[16];
    pthread_t                     rsplibThread;
@@ -200,8 +202,8 @@ int main(int argc, char** argv)
 #endif
 
    GList*                        clientList                     = NULL;
-   card64                        start                          = getMicroTime();
-   card64                        stop                           = 0;
+   unsigned long long            start                          = getMicroTime();
+   unsigned long long            stop                           = 0;
    int                           type                           = SOCK_STREAM;
    int                           protocol                       = IPPROTO_SCTP;
    unsigned short                port                           = 0;
@@ -227,7 +229,7 @@ int main(int argc, char** argv)
          protocol = IPPROTO_TCP;
       }
       else if(!(strncmp(argv[i], "-stop=" ,6))) {
-         stop = start + ((card64)1000000 * (card64)atol((char*)&argv[i][6]));
+         stop = start + ((unsigned long long)1000000 * (unsigned long long)atol((char*)&argv[i][6]));
       }
       else if(!(strncmp(argv[i], "-port=" ,6))) {
          port = atol((char*)&argv[i][6]);
@@ -437,7 +439,7 @@ int main(int argc, char** argv)
                                    (struct PoolElementDescriptor**)&pedArray,
                                    (unsigned int*)&pedStatusArray,
                                    1,
-                                   500000,
+                                   250000,
                                    (struct TagItem*)&tags);
 
          /* ====== Handle results of ext_select() ======================== */
@@ -490,8 +492,10 @@ int main(int argc, char** argv)
             }
             list = g_list_first(clientList);
          }
-         if(result < 0) {
-            perror("rspSessionSelect() failed");
+         if((result < 0) && (errno != EINTR)) {
+	    /* FIXME
+	    perror("rspSessionSelect() failed");
+            */
             break;
          }
 
@@ -502,6 +506,8 @@ int main(int argc, char** argv)
          }
       }
 
+      puts("Shutdown...");
+
       /* ====== Delete sessions and finally pool element ================= */
       list = g_list_first(clientList);
       while(list != NULL) {
