@@ -1,5 +1,5 @@
 /*
- *  $Id: examplepu.c,v 1.16 2004/12/16 16:16:59 dreibh Exp $
+ *  $Id: examplepu.c 732 2005-08-26 12:57:17Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -54,9 +54,9 @@
 
 
 struct SessionDescriptor* Session;
-card64                    OutCalls = 0;
-card64                    InBytes  = 0;
-card64                    OutBytes = 0;
+unsigned long long        OutCalls = 0;
+unsigned long long        InBytes  = 0;
+unsigned long long        OutBytes = 0;
 
 
 /* ###### Handle standard input ############################################# */
@@ -70,26 +70,37 @@ bool handleFailover(void* userData)
 /* ###### Handle standard input ############################################# */
 static void handleStdIn()
 {
-   char   buffer[8192];
-   size_t lineNumberLength;
-   size_t toSend;
+   char    buffer[8192];
+   size_t  lineNumberLength;
+   size_t  toSend;
+   ssize_t received;
 
    snprintf((char*)&buffer, sizeof(buffer),"%09lld ", ++OutCalls);
    lineNumberLength = strlen(buffer);
 
    setNonBlocking(STDIN_FILENO);
-   fgets((char*)&buffer[lineNumberLength], sizeof(buffer) - lineNumberLength, stdin);
+   received = read(STDIN_FILENO,
+                   (char*)&buffer[lineNumberLength],
+                   sizeof(buffer) - lineNumberLength - 1);
    setBlocking(STDIN_FILENO);
 
-   if(buffer[lineNumberLength] == 0x00) {
+   if(received <= 0) {
+      puts("<Ctrl-D>");
       sendBreak(true);
    }
-   toSend = strlen(buffer);
-   if(rspSessionWrite(Session, (char*)&buffer, toSend, NULL) <= 0) {
-      puts("Writing to pool element failed!");
-   }
    else {
-      OutBytes += toSend;
+      buffer[lineNumberLength + received] = 0x00;
+      if(lineNumberLength + received >= 1) {
+         buffer[lineNumberLength + received - 1] = 0x00;
+      }
+
+      toSend = strlen(buffer);
+      if(rspSessionWrite(Session, (char*)&buffer, toSend, NULL) <= 0) {
+         puts("Writing to pool element failed!");
+      }
+      else {
+         OutBytes += toSend;
+      }
    }
 }
 
@@ -122,10 +133,10 @@ static void handleAutoTimer()
 /* ###### Handle server reply ############################################### */
 static void handleServerReply()
 {
-   struct TagItem tags[16];
-   char           buffer[8193];
-   ssize_t        received;
-   static card64  counter = 1;
+   struct TagItem            tags[16];
+   char                      buffer[8193];
+   ssize_t                   received;
+   static unsigned long long counter = 1;
 
    tags[0].Tag = TAG_RspIO_PE_ID;
    tags[1].Tag = TAG_DONE;
@@ -142,12 +153,14 @@ static void handleServerReply()
       puts(buffer);
       fflush(stdout);
    }
+   /*
    else if(errno != EAGAIN) {
       printf("rspSessionRead failed: code %d\n", received);
    }
    else {
       puts("Message/Partial -- DAS SOLLTE NICHT ANGEZEIGT WERDEN!");
    }
+   */
 }
 
 
@@ -166,10 +179,10 @@ int main(int argc, char** argv)
    unsigned int              statusArray[1];
    fd_set                    readfdset;
    char*                     poolHandle    = "EchoPool";
-   card64                    autoInterval  = 0;
-   card64                    selectTimeout = 0;
-   card64                    nextAutoSend  = 250000;
-   card64                    now;
+   unsigned long long        autoInterval  = 0;
+   unsigned long long        selectTimeout = 0;
+   unsigned long long        nextAutoSend  = 250000;
+   unsigned long long        now;
    int                       result;
    int                       i, n;
 
@@ -181,7 +194,7 @@ int main(int argc, char** argv)
          poolHandle = (char*)&argv[i][4];
       }
       else if(!(strncmp(argv[i],"-auto=",6))) {
-         autoInterval = 1000 * (card64)atol((char*)&argv[i][6]);
+         autoInterval = 1000 * (unsigned long long)atol((char*)&argv[i][6]);
       }
 #ifdef ENABLE_CSP
       else if(!(strncmp(argv[i], "-identifier=", 12))) {
@@ -335,7 +348,7 @@ int main(int argc, char** argv)
             handleAutoTimer();
             nextAutoSend += autoInterval;
          }
-         if(result < 0) {
+         if((result < 0) && (errno != EINTR)) {
             perror("rspSelect() failed");
             break;
          }
