@@ -1,5 +1,5 @@
 /*
- *  $Id: loglevel.h,v 1.6 2004/11/13 03:24:13 dreibh Exp $
+ *  $Id: loglevel.h 687 2005-08-04 11:19:38Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -74,7 +74,7 @@ void setLogColor(const unsigned int color);
 
 
 #define stdlog (*gStdLog)
-#define logerror(text) fprintf(stdlog,"%s: %s\n",text,strerror(errno))
+#define logerror(text) fprintf(stdlog,"%s: %s\n", text, strerror(errno))
 
 
 #define LOG_BEGIN(prefix,c1,c2)              \
@@ -82,13 +82,16 @@ void setLogColor(const unsigned int color);
       loggingMutexLock();                    \
       setLogColor(c1);                       \
       printTimeStamp(stdlog);                \
-      fprintf(stdlog,"P%u.%x %s:%u %s() - ", \
+      setLogColor(c2);                       \
+      fprintf(stdlog,"P%u.%x %s:%u %s()\n",  \
               (unsigned int)getpid(),        \
               (unsigned int)pthread_self(),  \
               __FILE__,                      \
               __LINE__,                      \
               __FUNCTION__                   \
               );                             \
+      setLogColor(c1);                       \
+      printTimeStamp(stdlog);                \
       setLogColor(c2);                       \
       fputs(prefix,stdlog);
 
