@@ -1,5 +1,5 @@
 /*
- *  $Id: netutilities.c,v 1.44 2005/03/02 13:34:16 dreibh Exp $
+ *  $Id: netutilities.c 731 2005-08-25 17:17:42Z tuexen $
  *
  * RSerPool implementation.
  *
@@ -55,53 +55,12 @@
 #include <sys/sockio.h>
 #endif
 
+
 #ifdef SOLARIS
 #define CMSG_SPACE(len) (_CMSG_HDR_ALIGN(sizeof(struct cmsghdr)) + _CMSG_DATA_ALIGN(len))
 #define CMSG_LEN(len) (_CMSG_HDR_ALIGN(sizeof(struct cmsghdr)) + (len))
 #endif
 
-#ifdef HAVE_KERNEL_SCTP
-#ifndef HAVE_SCTP_CONNECTX
-const struct sockaddr* getBestScopedAddress(const struct sockaddr* addrs,
-                                            int                    addrcnt)
-{
-   const struct sockaddr* bestScopedAddress = addrs;
-   unsigned int           bestScope   = getScope(addrs);
-   unsigned int           newScope;
-   const struct sockaddr* a;
-   size_t                 i;
-
-   LOG_VERBOSE4
-   fputs("Finding best scope out of address set:\n", stdlog);
-   a = addrs;
-   for(i = 0;i < addrcnt;i++) {
-      fputs("   - ", stdlog);
-      fputaddress(a, true, stdlog);
-      fprintf(stdlog, ", scope=%u\n", getScope(a));
-      a = (const struct sockaddr*)((long)a + (long)getSocklen(a));
-   }
-   LOG_END
-
-   a = addrs;
-   for(i = 1;i < addrcnt;i++) {
-      a = (const struct sockaddr*)((long)a + (long)getSocklen(a));
-      newScope = getScope(a);
-      if(newScope > bestScope) {
-         bestScopedAddress = a;
-         bestScope   = newScope;
-      }
-   }
-
-   LOG_VERBOSE4
-   fputs("Using address ", stdlog);
-   fputaddress(bestScopedAddress, true, stdlog);
-   fprintf(stdlog, ", scope=%u\n", bestScope);
-   LOG_END
-   return(bestScopedAddress);
-}
-#endif
-#endif
-
 #ifdef HAVE_KERNEL_SCTP
 #ifndef HAVE_SCTP_CONNECTX
 #warning No sctp_connectx() available - Using only the first address!
@@ -141,8 +100,7 @@ ssize_t sctp_send(int                           sd,
 
    sri = (struct sctp_sndrcvinfo*)CMSG_DATA(cmsg);
    memcpy(sri, sinfo, sizeof(struct sctp_sndrcvinfo));
-
-   return(ext_sendmsg(sd, &msg, 0));
+   return(ext_sendmsg(sd, &msg, msg.msg_flags));
 }
 #endif
 
@@ -180,7 +138,7 @@ ssize_t sctp_sendx(int                           sd,
    sri = (struct sctp_sndrcvinfo*)CMSG_DATA(cmsg);
    memcpy(sri, sinfo, sizeof(struct sctp_sndrcvinfo));
 
-   return(ext_sendmsg(sd, &msg, 0));
+   return(ext_sendmsg(sd, &msg, msg.msg_flags));
 }
 #endif
 #endif
@@ -248,9 +206,9 @@ static bool filterAddress(union sockaddr_union* address,
 
 
 /* ###### Gather local addresses ######################################### */
-bool obtainLocalAddresses(union sockaddr_union** addressArray,
-                          size_t*                addresses,
-                          const unsigned int     flags)
+static bool obtainLocalAddresses(union sockaddr_union** addressArray,
+                                 size_t*                addresses,
+                                 const unsigned int     flags)
 {
    union sockaddr_union* localAddresses = NULL;
    struct sockaddr*         toUse;
@@ -431,9 +389,116 @@ bool obtainLocalAddresses(union sockaddr_union** addressArray,
 #define MAX_AUTOSELECT_PORT   60000
 
 
+#if 0
+/* ###### Convert mixed IPv4/IPv6 addresses to IPv6-format ############### */
+static struct sockaddr_in6* convertToIPv6(const struct sockaddr* addrs,
+                                          const int              addrcnt)
+{
+   union sockaddr_union* addrArray = (union sockaddr_union*)addrs;
+   struct sockaddr_in6*  newAddressArray;
+   int                   i;
+
+   newAddressArray = (struct sockaddr_in6*)malloc(sizeof(struct sockaddr_in6) * addrcnt);
+   if(newAddressArray == NULL) {
+      errno = ENOMEM;
+      return(NULL);
+   }
+
+   for(i = 0;i < addrcnt;i++) {
+      switch(addrArray->sa.sa_family) {
+         case AF_INET:
+            newAddressArray[i].sin6_family            = AF_INET6;
+            newAddressArray[i].sin6_flowinfo          = 0;
+            newAddressArray[i].sin6_port              = addrArray->in.sin_port;
+            newAddressArray[i].sin6_addr.s6_addr32[0] = 0x0000;
+            newAddressArray[i].sin6_addr.s6_addr32[1] = 0x0000;
+            newAddressArray[i].sin6_addr.s6_addr32[2] = htonl(0x0000ffff);
+            newAddressArray[i].sin6_addr.s6_addr32[3] = addrArray->in.sin_addr.s_addr;
+#ifdef HAVE_SOCKLEN_T
+            newAddressArray[i].sin6_socklen           = sizeof(struct sockaddr_in6);
+#endif
+            addrArray = (union sockaddr_union*)((long)addrArray + (long)sizeof(struct sockaddr_in));
+          break;
+         case AF_INET6:
+            memcpy((void*)&newAddressArray[i], addrArray, sizeof(struct sockaddr_in6));
+            addrArray = (union sockaddr_union*)((long)addrArray + (long)sizeof(struct sockaddr_in6));
+          break;
+         default:
+            free(newAddressArray);
+            errno = EPROTOTYPE;
+            return(NULL);
+          break;
+      }
+   }
+
+   for(i = 0;i < addrcnt;i++) {
+      printf("A%d=",i);
+      fputaddress((struct sockaddr*)&newAddressArray[i], true, stdout);
+      puts("");
+   }
+
+   return(newAddressArray);
+}
+#endif
+
+
+/* ###### Wrapper for sctp_bindx() ######################################## */
+static int my_sctp_bindx(int              sockfd,
+                         struct sockaddr* addrs,
+                         int              addrcnt,
+                         int              flags)
+{
+#if 0
+#warning Using Solaris-compatible sctp_bindx()!
+   union sockaddr_union* addrArray = (union sockaddr_union*)addrs;
+   struct sockaddr_in6*  newAddressArray;
+   int                   result;
+
+   result = sctp_bindx(sockfd, addrs, addrcnt, flags);
+   return(result);
+#else
+   return(sctp_bindx(sockfd, addrs, addrcnt, flags));
+#endif
+}
+
+
+/* ###### Wrapper for sctp_connectx() #################################### */
+static int my_sctp_connectx(int              sockfd,
+                            struct sockaddr* addrs,
+                            int              addrcnt)
+{
+#if 0
+#warning Using Solaris-compatible sctp_connectx()!
+   struct sockaddr_in6*  newAddressArray;
+   int                   result;
+
+result=-1;
+//   result = sctp_connectx(sockfd, addrs, addrcnt);
+//   printf("1: sctp_connectx() %d errno=%d   inprogress=%d\n",result,errno,EINPROGRESS);
+//   if((result < 0) && (errno == EPROTOTYPE)) {
+
+      newAddressArray = convertToIPv6(addrs, addrcnt);
+      if(newAddressArray) {
+         result = sctp_connectx(sockfd, (struct sockaddr*)newAddressArray, addrcnt);
+         printf("2: sctp_connectx() %d errno=%d   inprogress=%d\n",result,errno,EINPROGRESS);
+         free(newAddressArray);
+      }
+      else {
+         return(-1);
+      }
+//   }
+
+   return(result);
+#else
+   int result = sctp_connectx(sockfd, addrs, addrcnt);
+   return(result);
+#endif
+}
+
 
 /* ###### Unpack sockaddr blocks to sockaddr_union array ################## */
-union sockaddr_union* unpack_sockaddr(struct sockaddr* addrArray, const size_t addrs)
+union sockaddr_union* unpack_sockaddr(struct sockaddr* addrArray,
+                                      const size_t     addrs)
 {
    union sockaddr_union* newArray;
    size_t                i;
@@ -454,7 +519,6 @@ union sockaddr_union* unpack_sockaddr(struct sockaddr* addrArray, const size_t a
                LOG_ERROR
                fprintf(stderr, "ERROR: unpack_sockaddr() - Unknown address type #%d\n",
                        addrArray->sa_family);
-               fputs("IMPORTANT NOTE:\nThe standardizers have changed the socket API; the sockaddr_union array has been replaced by a variable-sized sockaddr_in/in6 blocks. Do not blame us for this change, send your complaints to the standardizers at sctp-impl@external.cisco.com!", stderr);
                LOG_END_FATAL
              break;
          }
@@ -465,7 +529,8 @@ union sockaddr_union* unpack_sockaddr(struct sockaddr* addrArray, const size_t a
 
 
 /* ###### Pack sockaddr_union array to sockaddr blocks #################### */
-struct sockaddr* pack_sockaddr_union(const union sockaddr_union* addrArray, const size_t addrs)
+struct sockaddr* pack_sockaddr_union(const union sockaddr_union* addrArray,
+                                     const size_t                addrs)
 {
    size_t           required = 0;
    size_t           i;
@@ -483,7 +548,6 @@ struct sockaddr* pack_sockaddr_union(const union sockaddr_union* addrArray, cons
             LOG_ERROR
             fprintf(stderr, "ERROR: pack_sockaddr_union() - Unknown address type #%d\n",
                     ((struct sockaddr*)&addrArray[i])->sa_family);
-            fputs("IMPORTANT NOTE:\nThe standardizers have changed the socket API; the sockaddr_union array has been replaced by a variable-sized sockaddr_in/in6 blocks. Do not blame us for this change, send your complaints to the standardizers at sctp-impl@external.cisco.com!", stderr);
             LOG_END_FATAL
           break;
       }
@@ -520,6 +584,7 @@ size_t getAddressesFromSocket(int sockfd, union sockaddr_union** addressArray)
    socklen_t            addressLength;
    ssize_t              addresses;
    ssize_t              result;
+   ssize_t              i;
 
    LOG_VERBOSE4
    fputs("Getting transport addresses from socket...\n",stdlog);
@@ -540,7 +605,7 @@ size_t getAddressesFromSocket(int sockfd, union sockaddr_union** addressArray)
          fputs("Successfully obtained address by getsockname()\n",stdlog);
          LOG_END
 
-         *addressArray = duplicateAddressArray((const union sockaddr_union*)&address,1);
+         *addressArray = duplicateAddressArray((const union sockaddr_union*)&address, 1);
          if(*addressArray != NULL) {
             addresses = 1;
          }
@@ -557,6 +622,14 @@ size_t getAddressesFromSocket(int sockfd, union sockaddr_union** addressArray)
       LOG_END
    }
 
+   LOG_VERBOSE4
+   fprintf(stdlog, "Obtained addresses: %u\n", addresses);
+   for(i = 0;i < addresses;i++) {
+      fputaddress((const struct sockaddr*)&addressArray[i], true, stdlog);
+      fputs("\n", stdlog);
+   }
+   LOG_END
+
    return((size_t)addresses);
 }
 
@@ -571,10 +644,7 @@ size_t gatherLocalAddresses(union sockaddr_union** addressArray)
    string2address(checkIPv6() ? "[::]" : "0.0.0.0", &anyAddress);
    sd = ext_socket(checkIPv6() ? AF_INET6 : AF_INET, SOCK_SEQPACKET, IPPROTO_SCTP);
    if(sd >= 0) {
-      /* The sctplib1 has a bug when binding to port 0. bindplus() avoids this
-         problem by selecting a port!
-         if(sd, (struct sockaddr*)&anyAddress, getSocklen(&anyAddress.sa)) == 0) { */
-      if(bindplus(sd, &anyAddress, 1)) {
+      if(ext_bind(sd, (struct sockaddr*)&anyAddress, getSocklen(&anyAddress.sa)) == 0) {
          addresses = getAddressesFromSocket(sd, addressArray);
       }
       ext_close(sd);
@@ -754,14 +824,14 @@ bool string2address(const char* string, union sockaddr_union* address)
    switch(ipv4address->sin_family) {
       case AF_INET:
          ipv4address->sin_port = htons(portNumber);
-#ifdef HAVE_SA_LEN
+#ifdef HAVE_SIN_LEN
          ipv4address->sin_len  = sizeof(struct sockaddr_in);
 #endif
        break;
       case AF_INET6:
          ipv6address->sin6_port = htons(portNumber);
-#ifdef HAVE_SA_LEN
-         ipv6address->sin_len  = sizeof(struct sockaddr_in6);
+#ifdef HAVE_SIN6_LEN
+         ipv6address->sin6_len  = sizeof(struct sockaddr_in6);
 #endif
        break;
       default:
@@ -891,6 +961,46 @@ unsigned int getScope(const struct sockaddr* address)
 }
 
 
+/* ###### Get one address of highest scope from address array ############ */
+const struct sockaddr* getBestScopedAddress(const struct sockaddr* addrs,
+                                            int                    addrcnt)
+{
+   const struct sockaddr* bestScopedAddress = addrs;
+   unsigned int           bestScope         = getScope(addrs);
+   unsigned int           newScope;
+   const struct sockaddr* a;
+   int                    i;
+
+   LOG_VERBOSE4
+   fputs("Finding best scope out of address set:\n", stdlog);
+   a = addrs;
+   for(i = 0;i < addrcnt;i++) {
+      fputs("   - ", stdlog);
+      fputaddress(a, true, stdlog);
+      fprintf(stdlog, ", scope=%u\n", getScope(a));
+      a = (const struct sockaddr*)((long)a + (long)getSocklen(a));
+   }
+   LOG_END
+
+   a = addrs;
+   for(i = 1;i < addrcnt;i++) {
+      a = (const struct sockaddr*)((long)a + (long)getSocklen(a));
+      newScope = getScope(a);
+      if(newScope > bestScope) {
+         bestScopedAddress = a;
+         bestScope   = newScope;
+      }
+   }
+
+   LOG_VERBOSE4
+   fputs("Using address ", stdlog);
+   fputaddress(bestScopedAddress, true, stdlog);
+   fprintf(stdlog, ", scope=%u\n", bestScope);
+   LOG_END
+   return(bestScopedAddress);
+}
+
+
 /* ###### Compare addresses ############################################## */
 int addresscmp(const struct sockaddr* a1, const struct sockaddr* a2, const bool port)
 {
@@ -988,7 +1098,7 @@ int addresscmp(const struct sockaddr* a1, const struct sockaddr* a2, const bool
 
 
 /* ###### Get port ####################################################### */
-uint16_t getPort(struct sockaddr* address)
+uint16_t getPort(const struct sockaddr* address)
 {
    if(address != NULL) {
       switch(address->sa_family) {
@@ -1036,7 +1146,7 @@ bool setPort(struct sockaddr* address, uint16_t port)
 
 
 /* ###### Get address family ############################################# */
-int getFamily(struct sockaddr* address)
+int getFamily(const struct sockaddr* address)
 {
    if(address != NULL) {
       return(address->sa_family);
@@ -1197,7 +1307,7 @@ bool bindplus(int                   sockfd,
          else {
             packedAddresses = pack_sockaddr_union(addressArray, addresses);
             if(packedAddresses) {
-               result = sctp_bindx(sockfd, packedAddresses, addresses, SCTP_BINDX_ADD_ADDR);
+               result = my_sctp_bindx(sockfd, packedAddresses, addresses, SCTP_BINDX_ADD_ADDR);
                free(packedAddresses);
             }
             else {
@@ -1251,19 +1361,22 @@ int sendtoplus(int                      sockfd,
    char*                  p;
 
    LOG_VERBOSE4
-   fprintf(stdlog, "sendmsg(%d/A%u, %u bytes) PPID=$%08x streamID=%u toaddrs=%p toaddrcnt=%u...\n",
-           sockfd, (unsigned int)assocID, (unsigned int)length, ppid, streamID, toaddrs, (unsigned int)toaddrcnt);
+   fprintf(stdlog, "sendmsg(%d/A%u, %u bytes) PPID=$%08x streamID=%u flags=$%x toaddrs=%p toaddrcnt=%u...\n",
+           sockfd, (unsigned int)assocID, (unsigned int)length, ppid, streamID, flags,toaddrs, (unsigned int)toaddrcnt);
    LOG_END
 
    setNonBlocking(sockfd);
    if((assocID != 0) || (ppid != 0) || (streamID != 0)) {
+      memset(&sri, 0, sizeof(sri));
       sri.sinfo_assoc_id   = assocID;
       sri.sinfo_stream     = streamID;
       sri.sinfo_ppid       = htonl(ppid);
-      sri.sinfo_flags      = flags;
+      /* --- Already resetted to 0 ---
+      sri.sinfo_flags      = 0;
       sri.sinfo_ssn        = 0;
       sri.sinfo_tsn        = 0;
       sri.sinfo_context    = 0;
+      */
       sri.sinfo_timetolive = timeToLive;
 
       if(toaddrs) {
@@ -1520,6 +1633,7 @@ static bool multicastGroupMgt(int              sockfd,
    struct ip_mreq   mreq;
    struct ifreq     ifr;
    struct ipv6_mreq mreq6;
+   int              result;
 
    if(address->sa_family == AF_INET) {
       mreq.imr_multiaddr = ((struct sockaddr_in*)address)->sin_addr;
@@ -1533,9 +1647,12 @@ static bool multicastGroupMgt(int              sockfd,
       else {
          memset((char*)&mreq.imr_interface, 0, sizeof(mreq.imr_interface));
       }
-      return(ext_setsockopt(sockfd,IPPROTO_IP,
-                            add ? IP_ADD_MEMBERSHIP : IP_DROP_MEMBERSHIP,
-                            &mreq, sizeof(mreq)) == 0);
+      /* BSD sets errno to EADDRINUSE if socket has already been joined to
+         the multicast group. This is no real error. */
+      result = ext_setsockopt(sockfd,IPPROTO_IP,
+                              add ? IP_ADD_MEMBERSHIP : IP_DROP_MEMBERSHIP,
+                              &mreq, sizeof(mreq));
+      return((result == 0) || (errno == EADDRINUSE));
    }
    else if(address->sa_family == AF_INET6) {
       memcpy((char*)&mreq6.ipv6mr_multiaddr,
@@ -1547,9 +1664,12 @@ static bool multicastGroupMgt(int              sockfd,
       else {
          mreq6.ipv6mr_interface = 0;
       }
-      return(ext_setsockopt(sockfd,IPPROTO_IPV6,
-                            add ? IPV6_JOIN_GROUP : IPV6_LEAVE_GROUP,
-                            &mreq6, sizeof(mreq6)) == 0);
+      result = ext_setsockopt(sockfd,IPPROTO_IPV6,
+                              add ? IPV6_JOIN_GROUP : IPV6_LEAVE_GROUP,
+                              &mreq6, sizeof(mreq6));
+      /* BSD sets errno to EADDRINUSE if socket has already been joined to
+         the multicast group. This is no real error. */
+      return((result == 0) || (errno == EADDRINUSE));
    }
    CHECK(false);
    return(false);
@@ -1608,7 +1728,7 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
       if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_RTOINFO, &rtoinfo, (socklen_t)sizeof(rtoinfo)) < 0) {
          result = false;
          LOG_WARNING
-         perror("SCTP_RTOINFO");
+         logerror("SCTP_RTOINFO");
          fprintf(stdlog, "Unable to update RTO info of socket %d, assoc %u\n", sockfd, (unsigned int)assocID);
          LOG_END
       }
@@ -1646,7 +1766,7 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
       if(ext_setsockopt(sockfd, IPPROTO_SCTP, SCTP_ASSOCINFO, &assocParams, (socklen_t)sizeof(assocParams)) < 0) {
          result = false;
          LOG_WARNING
-         perror("SCTP_ASSOCINFO");
+         logerror("SCTP_ASSOCINFO");
          fprintf(stdlog, "Unable to update Assoc info of socket %d, assoc %u\n", sockfd, (unsigned int)assocID);
          LOG_END
       }
@@ -1670,22 +1790,37 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
    n = getpaddrsplus(sockfd, assocID, &addrs);
    if(n >= 0) {
       for(i = 0;i < n;i++) {
+         memset(&peerParams, 0, sizeof(peerParams));
          peerParams.spp_assoc_id = assocID;
          memcpy((void*)&(peerParams.spp_address),
                 (const void*)&addrs[i], sizeof(union sockaddr_union));
          size = (socklen_t)sizeof(peerParams);
 
-         if(sctp_opt_info(sockfd, assocID, SCTP_GET_PEER_ADDR_INFO,
+         if(sctp_opt_info(sockfd, assocID, SCTP_PEER_ADDR_PARAMS,
                           (void*)&peerParams, &size) == 0) {
             LOG_VERBOSE3
             fputs("Old peer parameters for address ", stdlog);
             fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
+#ifdef HAVE_SPP_FLAGS
+            fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d flags=$%x\n",
+                    sockfd, (unsigned int)assocID,
+                    peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt, peerParams.spp_flags);
+#else
             fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d\n",
                     sockfd, (unsigned int)assocID,
                     peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt);
+#endif
             LOG_END
 
             peerParams.spp_hbinterval = tagListGetData(tags, TAG_TuneSCTP_Heartbeat,  peerParams.spp_hbinterval);
+#ifdef HAVE_SPP_FLAGS
+            if(peerParams.spp_hbinterval > 0) {
+               peerParams.spp_flags |= SPP_HB_ENABLE;
+            }
+            else {
+               peerParams.spp_flags |= SPP_HB_DISABLE;
+            }
+#endif
             peerParams.spp_pathmaxrxt = tagListGetData(tags, TAG_TuneSCTP_PathMaxRXT, peerParams.spp_pathmaxrxt);;
 
             if(sctp_opt_info(sockfd, 0, SCTP_PEER_ADDR_PARAMS,
@@ -1701,9 +1836,15 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
                LOG_VERBOSE2
                fputs("New peer parameters for address ", stdlog);
                fputaddress((struct sockaddr*)&(peerParams.spp_address), false, stdlog);
-               fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d\n",
+#ifdef HAVE_SPP_FLAGS
+                fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d flags=$%x\n",
+                        sockfd, (unsigned int)assocID,
+                        peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt, peerParams.spp_flags);
+#else
+                fprintf(stdlog, " on socket %d, assoc %u: hb=%d maxrxt=%d\n",
                         sockfd, (unsigned int)assocID,
                         peerParams.spp_hbinterval, peerParams.spp_pathmaxrxt);
+#endif
                LOG_END
             }
          }
@@ -1729,13 +1870,33 @@ bool tuneSCTP(int sockfd, sctp_assoc_t assocID, struct TagItem* tags)
 }
 
 
-/* ###### Create socket and establish connection ############################ */
-int establish(const int             socketDomain,
-              const int             socketType,
-              const int             socketProtocol,
-              union sockaddr_union* addressArray,
-              const size_t          addresses,
-              const unsigned long long          timeout)
+/* ###### Establish connection ########################################### */
+int connectplus(int                   sockfd,
+                union sockaddr_union* addressArray,
+                const size_t          addresses)
+{
+   struct sockaddr* packedAddresses;
+   int              result;
+
+   packedAddresses = pack_sockaddr_union(addressArray, addresses);
+   if(packedAddresses) {
+      result = my_sctp_connectx(sockfd, packedAddresses, addresses);
+      free(packedAddresses);
+   }
+   else {
+      result = -ENOMEM;
+   }
+   return(result);
+}
+
+
+/* ###### Create socket and establish connection ######################### */
+int establish(const int                socketDomain,
+              const int                socketType,
+              const int                socketProtocol,
+              union sockaddr_union*    addressArray,
+              const size_t             addresses,
+              const unsigned long long timeout)
 {
    fd_set               fdset;
    struct timeval       to;
@@ -1769,7 +1930,7 @@ int establish(const int             socketDomain,
       if(socketProtocol == IPPROTO_SCTP) {
          packedAddresses = pack_sockaddr_union(addressArray, addresses);
          if(packedAddresses) {
-            result = sctp_connectx(sockfd, packedAddresses, addresses);
+            result = my_sctp_connectx(sockfd, packedAddresses, addresses);
             free(packedAddresses);
          }
          else {
@@ -1856,16 +2017,16 @@ size_t getladdrsplus(const int              fd,
 {
    struct sockaddr* packedAddresses;
 #ifdef SOLARIS
-   size_t addrs = sctp_getladdrs(fd, assocID, (void**)&packedAddresses);
+   int addrs = sctp_getladdrs(fd, assocID, (void**)&packedAddresses);
 #else
-   size_t addrs = sctp_getladdrs(fd, assocID, &packedAddresses);
+   int addrs = sctp_getladdrs(fd, assocID, &packedAddresses);
 #endif
 #ifdef LINUX
 #ifdef HAVE_KERNEL_SCTP
    uint16_t port;
-   size_t   i;
 #endif
 #endif
+   size_t i;
 
    if(addrs > 0) {
 #ifdef LINUX
@@ -1889,10 +2050,21 @@ size_t getladdrsplus(const int              fd,
       }
 #endif
 #endif
+
       *addressArray = unpack_sockaddr(packedAddresses, addrs);
       sctp_freeladdrs(packedAddresses);
+
+      LOG_VERBOSE5
+      fprintf(stdlog, "getladdrsplus() - Number of addresses: %u\n", (unsigned int)addrs);
+      for(i = 0;i < addrs;i++) {
+         fprintf(stdlog, " - #%u: ", (unsigned int)i);
+         fputaddress(&(*addressArray)[i].sa, true, stdlog);
+         fputs("\n", stdlog);
+      }
+      LOG_END
+      return((size_t)addrs);
    }
-   return(addrs);
+   return(0);
 }
 
 
@@ -1903,24 +2075,25 @@ size_t getpaddrsplus(const int              fd,
 {
    struct sockaddr* packedAddresses;
 #ifdef SOLARIS
-   size_t addrs = sctp_getpaddrs(fd, assocID, (void **)&packedAddresses);
+   int addrs = sctp_getpaddrs(fd, assocID, (void **)&packedAddresses);
 #else
-   size_t addrs = sctp_getpaddrs(fd, assocID, &packedAddresses);
+   int addrs = sctp_getpaddrs(fd, assocID, &packedAddresses);
 #endif
    if(addrs > 0) {
       *addressArray = unpack_sockaddr(packedAddresses, addrs);
       sctp_freepaddrs(packedAddresses);
+      return((size_t)addrs);
    }
-   return(addrs);
+   return(0);
 }
 
 
 /* ###### Convert byte order of 64 bit value ############################# */
-static uint64_t byteswap64(const uint64_t x)
+static uint64_t byteswap64(const uint64_t value)
 {
 #if (__BYTE_ORDER == __LITTLE_ENDIAN)
-   const uint32_t a = (uint32_t)(x >> 32);
-   const uint32_t b = (uint32_t)(x & 0xffffffff);
+   const uint32_t a = (uint32_t)(value >> 32);
+   const uint32_t b = (uint32_t)(value & 0xffffffff);
    return( (int64_t)((a << 24) | ((a & 0x0000ff00) << 8) |
            ((a & 0x00ff0000) >> 8) | (a >> 24)) |
            ((int64_t)((b << 24) | ((b & 0x0000ff00) << 8) |
