@@ -1,5 +1,5 @@
 /*
- *  $Id: netutilities.h,v 1.16 2004/11/13 03:24:13 dreibh Exp $
+ *  $Id: netutilities.h 680 2005-08-03 10:40:27Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -194,7 +194,7 @@ int addresscmp(const struct sockaddr* a1, const struct sockaddr* a2, const bool
   * @param address sockaddr structure.
   * @return Port or 0 in case of error.
   */
-uint16_t getPort(struct sockaddr* address);
+uint16_t getPort(const struct sockaddr* address);
 
 /**
   * Set port in sockaddr structure.
@@ -211,7 +211,7 @@ bool setPort(struct sockaddr* address, const uint16_t port);
   * @param address sockaddr structure.
   * @return Address family
   */
-int getFamily(struct sockaddr* address);
+int getFamily(const struct sockaddr* address);
 
 /**
   * Get relative scope of address. This function supports IPv4 and IPv6
@@ -222,6 +222,15 @@ int getFamily(struct sockaddr* address);
   */
 unsigned int getScope(const struct sockaddr* address);
 
+/**
+  * Get one address of highest scope from address array.
+  *
+  * @param addrs Address array.
+  * @param addrcnt Address count.
+  * @return Selected address.
+  */
+const struct sockaddr* getBestScopedAddress(const struct sockaddr* addrs,
+                                            int                    addrcnt);
 
 
 /**
@@ -300,6 +309,18 @@ bool bindplus(int                   sockfd,
               union sockaddr_union* addressArray,
               const size_t          addresses);
 
+/**
+  * Establish a connection.
+  *
+  * @param sockfd Socket descriptor.
+  * @param addressArray Destination address or NULL for connection-oriented socket.
+  * @param addresses Length of destination address or 0 if not given.
+  * @return 0 in case of success; error code <0 otherwise.
+  */
+int connectplus(int                   sockfd,
+                union sockaddr_union* addressArray,
+                const size_t          addresses);
+
 /**
   * Wrapper for sendmsg() with timeout and support for SCTP parameters.
   *
