@@ -1,6 +1,6 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2005 by Thomas Dreibholz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,21 +29,42 @@ void ST_CLASS(peerListIndexStorageNodePrint)(const void *nodePtr, FILE* fd)
 
 
 /* ###### Comparison ##################################################### */
-int ST_CLASS(peerListIndexStorageNodeComparison)(const void* nodePtr1, const void* nodePtr2)
+int ST_CLASS(peerListIndexStorageNodeComparison)(const void* nodePtr1,
+                                                 const void* nodePtr2)
 {
    const struct ST_CLASS(PeerListNode)* node1 = (struct ST_CLASS(PeerListNode)*)nodePtr1;
    const struct ST_CLASS(PeerListNode)* node2 = (struct ST_CLASS(PeerListNode)*)nodePtr2;
+   int                                  result;
 
+   /* One of the nodes has an undefined ID, i.e. static entry.
+      => Only comparison by address overlap possible */
+   if((node1->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER) ||
+      (node2->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER)) {
+      result = transportAddressBlockOverlapComparison(node1->AddressBlock,
+                                                      node2->AddressBlock);
+      if(result == 0) {
+         return(0);
+      }
+      /* No overlap -> continue comparison */
+   }
+
+   /* Compare IDs */
    if(node1->Identifier < node2->Identifier) {
       return(-1);
    }
    else if(node1->Identifier > node2->Identifier) {
       return(1);
    }
-   if(node1->Identifier == 0) {
-      return(transportAddressBlockComparison(node1->AddressBlock,
-                                             node2->AddressBlock));
+
+   /* The IDs are equal. If both IDs are undefined, we compare
+      the addresses completely to get a sorting order. */
+   if((node1->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER) &&
+      (node2->Identifier == UNDEFINED_REGISTRAR_IDENTIFIER)) {
+      result = transportAddressBlockComparison(node1->AddressBlock,
+                                               node2->AddressBlock);
+      return(result);
    }
+
    return(0);
 }
 
@@ -58,7 +79,8 @@ void ST_CLASS(peerListTimerStorageNodePrint)(const void* nodePtr, FILE* fd)
 
 
 /* ###### Comparison ##################################################### */
-int ST_CLASS(peerListTimerStorageNodeComparison)(const void *nodePtr1, const void *nodePtr2)
+int ST_CLASS(peerListTimerStorageNodeComparison)(const void *nodePtr1,
+                                                 const void *nodePtr2)
 {
    const struct ST_CLASS(PeerListNode)* node1 =
       ST_CLASS(getPeerListNodeFromPeerListTimerStorageNode)((void*)nodePtr1);
@@ -71,23 +93,14 @@ int ST_CLASS(peerListTimerStorageNodeComparison)(const void *nodePtr1, const voi
    else if(node1->TimerTimeStamp > node2->TimerTimeStamp) {
       return(1);
    }
-   if(node1->Identifier < node2->Identifier) {
-      return(-1);
-   }
-   else if(node1->Identifier > node2->Identifier) {
-      return(1);
-   }
-   if(node1->Identifier == 0) {
-      return(transportAddressBlockComparison(node1->AddressBlock,
-                                             node2->AddressBlock));
-   }
-   return(0);
+   /* Same timestamps => use regular comparison */
+   return(ST_CLASS(peerListIndexStorageNodeComparison)(nodePtr1, nodePtr2));
 }
 
 
 /* ###### Initialize ##################################################### */
 void ST_CLASS(peerListNodeNew)(struct ST_CLASS(PeerListNode)* peerListNode,
-                               const RegistrarIdentifierType       identifier,
+                               const RegistrarIdentifierType  identifier,
                                const unsigned int             flags,
                                struct TransportAddressBlock*  transportAddressBlock)
 {
@@ -98,9 +111,9 @@ void ST_CLASS(peerListNodeNew)(struct ST_CLASS(PeerListNode)* peerListNode,
 
    peerListNode->Identifier          = identifier;
    peerListNode->Flags               = flags;
+   peerListNode->OwnershipChecksum   = INITIAL_HANDLESPACE_CHECKSUM;
 
    peerListNode->LastUpdateTimeStamp = 0;
-
    peerListNode->TimerCode           = 0;
    peerListNode->TimerTimeStamp      = 0;
 
@@ -151,6 +164,14 @@ int ST_CLASS(peerListNodeUpdate)(struct ST_CLASS(PeerListNode)*       peerListNo
 }
 
 
+/* ###### Get ownership checksum ######################################### */
+HandlespaceChecksumType ST_CLASS(peerListNodeGetOwnershipChecksum)(
+                           const struct ST_CLASS(PeerListNode)* peerListNode)
+{
+   return(handlespaceChecksumFinish(peerListNode->OwnershipChecksum));
+}
+
+
 /* ###### Get textual description ######################################## */
 void ST_CLASS(peerListNodeGetDescription)(
         const struct ST_CLASS(PeerListNode)* peerListNode,
@@ -161,9 +182,10 @@ void ST_CLASS(peerListNodeGetDescription)(
    char transportAddressDescription[1024];
 
    snprintf(buffer, bufferSize,
-            "$%08x upd=%llu flgs=",
+            "$%08x upd=%llu chsum=$%08x flags=",
             peerListNode->Identifier,
-            peerListNode->LastUpdateTimeStamp);
+            peerListNode->LastUpdateTimeStamp,
+            (unsigned int)handlespaceChecksumFinish(peerListNode->OwnershipChecksum));
    if(!(peerListNode->Flags & PLNF_DYNAMIC)) {
       safestrcat(buffer, "static", bufferSize);
    }
