@@ -1,6 +1,6 @@
 /*
  * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004 by Thomas Dreibholz
+ * Copyright (C) 2004-2005 by Thomas Dreibholz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,42 +44,43 @@ struct ST_CLASS(PoolNode);
 
 struct ST_CLASS(PoolElementNode)
 {
-   struct STN_CLASSNAME          PoolElementSelectionStorageNode;
-   struct STN_CLASSNAME          PoolElementIndexStorageNode;
-   struct STN_CLASSNAME          PoolElementTimerStorageNode;
-   struct STN_CLASSNAME          PoolElementConnectionStorageNode;
-   struct STN_CLASSNAME          PoolElementOwnershipStorageNode;
-   struct ST_CLASS(PoolNode)*    OwnerPoolNode;
+   struct STN_CLASSNAME               PoolElementSelectionStorageNode;
+   struct STN_CLASSNAME               PoolElementIndexStorageNode;
+   struct STN_CLASSNAME               PoolElementTimerStorageNode;
+   struct STN_CLASSNAME               PoolElementConnectionStorageNode;
+   struct STN_CLASSNAME               PoolElementOwnershipStorageNode;
+   struct ST_CLASS(PoolNode)*         OwnerPoolNode;
 
-   PoolElementIdentifierType     Identifier;
+   HandlespaceChecksumAccumulatorType Checksum;
+   PoolElementIdentifierType          Identifier;
    RegistrarIdentifierType            HomeRegistrarIdentifier;
-   unsigned int                  RegistrationLife;
-   struct PoolPolicySettings     PolicySettings;
+   unsigned int                       RegistrationLife;
+   struct PoolPolicySettings          PolicySettings;
 
-   PoolElementSeqNumberType      SeqNumber;
-   PoolElementSeqNumberType      RoundCounter;
-   unsigned int                  VirtualCounter;
-   unsigned int                  Degradation;
-   unsigned int                  UnreachabilityReports;
-   unsigned long long            SelectionCounter;
-   unsigned long long            LastUpdateTimeStamp;
+   PoolElementSeqNumberType           SeqNumber;
+   PoolElementSeqNumberType           RoundCounter;
+   unsigned int                       VirtualCounter;
+   unsigned int                       Degradation;
+   unsigned int                       UnreachabilityReports;
+   unsigned long long                 SelectionCounter;
+   unsigned long long                 LastUpdateTimeStamp;
 
-   unsigned int                  TimerCode;
-   unsigned long long            TimerTimeStamp;
+   unsigned int                       TimerCode;
+   unsigned long long                 TimerTimeStamp;
 
-   int                           ConnectionSocketDescriptor;
-   sctp_assoc_t                  ConnectionAssocID;
+   int                                ConnectionSocketDescriptor;
+   sctp_assoc_t                       ConnectionAssocID;
 
-   struct TransportAddressBlock* UserTransport;
-   struct TransportAddressBlock* RegistratorTransport;
+   struct TransportAddressBlock*      UserTransport;
+   struct TransportAddressBlock*      RegistratorTransport;
 
-   void*                         UserData;
+   void*                              UserData;
 };
 
 
 void ST_CLASS(poolElementNodeNew)(struct ST_CLASS(PoolElementNode)* poolElementNode,
                                   const PoolElementIdentifierType   identifier,
-                                  const RegistrarIdentifierType          homeRegistrarIdentifier,
+                                  const RegistrarIdentifierType     homeRegistrarIdentifier,
                                   const unsigned int                registrationLife,
                                   const struct PoolPolicySettings*  pps,
                                   struct TransportAddressBlock*     userTransport,
@@ -96,6 +97,8 @@ void ST_CLASS(poolElementNodePrint)(
         struct ST_CLASS(PoolElementNode)* poolElementNode,
         FILE*                             fd,
         const unsigned int                fields);
+HandlespaceChecksumAccumulatorType ST_CLASS(poolElementNodeComputeChecksum)(
+                                      const struct ST_CLASS(PoolElementNode)* poolElementNode);
 int ST_CLASS(poolElementNodeUpdate)(struct ST_CLASS(PoolElementNode)*       poolElementNode,
                                     const struct ST_CLASS(PoolElementNode)* source);
 struct ST_CLASS(PoolElementNode)* ST_CLASS(getPoolElementNodeFromPoolElementSelectionStorageNode)(void* node);
