@@ -23,7 +23,7 @@
 /* ###### Initialize ##################################################### */
 void ST_CLASS(poolElementNodeNew)(struct ST_CLASS(PoolElementNode)* poolElementNode,
                                   const PoolElementIdentifierType   identifier,
-                                  const RegistrarIdentifierType          homeRegistrarIdentifier,
+                                  const RegistrarIdentifierType     homeRegistrarIdentifier,
                                   const unsigned int                registrationLife,
                                   const struct PoolPolicySettings*  pps,
                                   struct TransportAddressBlock*     userTransport,
@@ -39,8 +39,9 @@ void ST_CLASS(poolElementNodeNew)(struct ST_CLASS(PoolElementNode)* poolElementN
 
    poolElementNode->OwnerPoolNode               = NULL;
 
+   poolElementNode->Checksum                    = INITIAL_HANDLESPACE_CHECKSUM;
    poolElementNode->Identifier                  = identifier;
-   poolElementNode->HomeRegistrarIdentifier            = homeRegistrarIdentifier;
+   poolElementNode->HomeRegistrarIdentifier     = homeRegistrarIdentifier;
    poolElementNode->RegistrationLife            = registrationLife;
    poolElementNode->PolicySettings              = *pps;
 
@@ -75,6 +76,7 @@ void ST_CLASS(poolElementNodeDelete)(struct ST_CLASS(PoolElementNode)* poolEleme
    CHECK(!STN_METHOD(IsLinked)(&poolElementNode->PoolElementOwnershipStorageNode));
    CHECK(!STN_METHOD(IsLinked)(&poolElementNode->PoolElementConnectionStorageNode));
 
+   poolElementNode->Checksum                    = 0;
    poolElementNode->RegistrationLife            = 0;
    poolElementNode->OwnerPoolNode               = NULL;
    poolElementNode->SeqNumber                   = 0;
@@ -113,37 +115,50 @@ void ST_CLASS(poolElementNodeGetDescription)(
    char transportAddressDescription[1024];
 
    snprintf(buffer, bufferSize, "$%08x", poolElementNode->Identifier);
+   if(fields & (PENPO_CONNECTION|PENPO_CHECKSUM|PENPO_HOME_PR|PENPO_REGLIFE|PENPO_UR_REPORTS|PENPO_LASTUPDATE)) {
+      safestrcat(buffer, "\n     ", bufferSize);
+   }
    if(fields & PENPO_CONNECTION) {
-      snprintf((char*)&tmp, sizeof(tmp), " c=(S%d,A%u)",
+      snprintf((char*)&tmp, sizeof(tmp), "c=(S%d,A%u) ",
                poolElementNode->ConnectionSocketDescriptor,
                (unsigned int)poolElementNode->ConnectionAssocID);
       safestrcat(buffer, tmp, bufferSize);
    }
-   if(fields & PENPO_HOME_NS) {
-      snprintf((char*)&tmp, sizeof(tmp), " home=$%08x",
+   if(fields & PENPO_CHECKSUM) {
+      snprintf((char*)&tmp, sizeof(tmp), "chsum=$%08x ",
+               (unsigned int)handlespaceChecksumFinish(poolElementNode->Checksum));
+      safestrcat(buffer, tmp, bufferSize);
+   }
+   if(fields & PENPO_HOME_PR) {
+      snprintf((char*)&tmp, sizeof(tmp), "home=$%08x ",
                poolElementNode->HomeRegistrarIdentifier);
       safestrcat(buffer, tmp, bufferSize);
    }
    if(fields & PENPO_REGLIFE) {
-      snprintf((char*)&tmp, sizeof(tmp), " life=%ums",
+      snprintf((char*)&tmp, sizeof(tmp), "life=%ums ",
                poolElementNode->RegistrationLife);
       safestrcat(buffer, tmp, bufferSize);
    }
    if(fields & PENPO_UR_REPORTS) {
-      snprintf((char*)&tmp, sizeof(tmp), " ur=%u",
+      snprintf((char*)&tmp, sizeof(tmp), "ur=%u ",
                poolElementNode->UnreachabilityReports);
       safestrcat(buffer, tmp, bufferSize);
    }
+   if(fields & PENPO_LASTUPDATE) {
+      snprintf((char*)&tmp, sizeof(tmp), "upd=%llu ",
+               poolElementNode->LastUpdateTimeStamp);
+      safestrcat(buffer, tmp, bufferSize);
+   }
    if(fields & PENPO_POLICYINFO) {
       poolPolicySettingsGetDescription(&poolElementNode->PolicySettings,
                                        (char*)&poolPolicySettingsDescription,
                                        sizeof(poolPolicySettingsDescription));
-      snprintf((char*)&tmp, sizeof(tmp), "   [%s]",
+      snprintf((char*)&tmp, sizeof(tmp), "\n     %s",
                poolPolicySettingsDescription);
       safestrcat(buffer, tmp, bufferSize);
    }
    if(fields & PENPO_POLICYSTATE) {
-      snprintf((char*)&tmp, sizeof(tmp), "   seq=%llu val=%llu rd=%u vrt=%u deg=$%x {sel=%llu s/w=%1.1f}",
+      snprintf((char*)&tmp, sizeof(tmp), "\n     seq=%llu val=%llu rd=%u vrt=%u deg=$%x {sel=%llu s/w=%1.1f}",
                (unsigned long long)poolElementNode->SeqNumber,
                poolElementNode->PoolElementSelectionStorageNode.Value,
                poolElementNode->RoundCounter,
@@ -153,11 +168,6 @@ void ST_CLASS(poolElementNodeGetDescription)(
                (double)poolElementNode->SelectionCounter / (double)poolElementNode->PolicySettings.Weight);
       safestrcat(buffer, tmp, bufferSize);
    }
-   if(fields & PENPO_LASTUPDATE) {
-      snprintf((char*)&tmp, sizeof(tmp), "   upd=%llu",
-               poolElementNode->LastUpdateTimeStamp);
-      safestrcat(buffer, tmp, bufferSize);
-   }
    if((fields & PENPO_USERTRANSPORT) &&
       (poolElementNode->UserTransport->Addresses > 0)) {
       transportAddressBlockGetDescription(poolElementNode->UserTransport,
@@ -192,6 +202,39 @@ void ST_CLASS(poolElementNodePrint)(
 }
 
 
+/* ###### Compute PE checksum ############################################ */
+HandlespaceChecksumAccumulatorType ST_CLASS(poolElementNodeComputeChecksum)(
+                                      const struct ST_CLASS(PoolElementNode)* poolElementNode)
+{
+   HandlespaceChecksumAccumulatorType checksum   = INITIAL_HANDLESPACE_CHECKSUM;
+   PoolElementIdentifierType          identifier = htonl(poolElementNode->Identifier);
+
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&poolElementNode->OwnerPoolNode->Handle.Handle,
+                                         poolElementNode->OwnerPoolNode->Handle.Size);
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&identifier,
+                                         sizeof(identifier));
+
+/*
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&poolElementNode->HomeRegistrarIdentifier,
+                                         sizeof(poolElementNode->HomeRegistrarIdentifier));
+
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&poolElementNode->PolicySettings.Weight,
+                                         sizeof(poolElementNode->PolicySettings.Weight));
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&poolElementNode->PolicySettings.Load,
+                                         sizeof(poolElementNode->PolicySettings.Load));
+   checksum = handlespaceChecksumCompute(checksum,
+                                         (const char*)&poolElementNode->PolicySettings.LoadDegradation,
+                                         sizeof(poolElementNode->PolicySettings.LoadDegradation));
+*/
+   return(checksum);
+}
+
+
 /* ###### Update ######################################################### */
 int ST_CLASS(poolElementNodeUpdate)(struct ST_CLASS(PoolElementNode)*       poolElementNode,
                                     const struct ST_CLASS(PoolElementNode)* source)
