@@ -62,7 +62,8 @@ extern "C" {
 #define RSPERR_INVALID_POOL_HANDLE           0xf00a
 #define RSPERR_INVALID_ADDRESSES             0xf00b
 #define RSPERR_INVALID_REGISTRATOR           0xf00c
-#define RSPERR_NO_USABLE_ADDRESSES           0xf00d
+#define RSPERR_NO_USABLE_ASAP_ADDRESSES      0xf00d
+#define RSPERR_NO_USABLE_USER_ADDRESSES      0xf00e
 
 
 const char* rserpoolErrorGetDescription(const unsigned int error);
