@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessage.c,v 1.13 2004/11/19 16:42:47 dreibh Exp $
+ *  $Id: rserpoolmessage.c 697 2005-08-08 09:26:18Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -103,10 +103,20 @@ void rserpoolMessageClearAll(struct RSerPoolMessage* message)
    if(message != NULL) {
       if((message->PoolElementPtr) && (message->PoolElementPtrAutoDelete)) {
          ST_CLASS(poolElementNodeDelete)(message->PoolElementPtr);
+         transportAddressBlockDelete(message->PoolElementPtr->UserTransport);
+         free(message->PoolElementPtr->UserTransport);
+         message->PoolElementPtr->UserTransport = NULL;
+         if(message->PoolElementPtr->RegistratorTransport) {
+            transportAddressBlockDelete(message->PoolElementPtr->RegistratorTransport);
+            free(message->PoolElementPtr->RegistratorTransport);
+            message->PoolElementPtr->RegistratorTransport = NULL;
+         }
          free(message->PoolElementPtr);
+         message->PoolElementPtr = NULL;
       }
       if((message->CookiePtr) && (message->CookiePtrAutoDelete)) {
          free(message->CookiePtr);
+         message->CookiePtr = NULL;
       }
       if(message->TransportAddressBlockListPtrAutoDelete) {
          transportAddressBlock = message->TransportAddressBlockListPtr;
@@ -116,15 +126,23 @@ void rserpoolMessageClearAll(struct RSerPoolMessage* message)
             free(transportAddressBlock);
             transportAddressBlock = nextTransportAddressBlock;
          }
+         message->TransportAddressBlockListPtr = NULL;
       }
       message->TransportAddressBlockListPtr = NULL;
       if(message->PoolElementPtrArrayAutoDelete) {
-         CHECK(message->PoolElementPtrArraySize < MAX_MAX_NAME_RESOLUTION_ITEMS);
+         CHECK(message->PoolElementPtrArraySize < MAX_MAX_HANDLE_RESOLUTION_ITEMS);
          for(i = 0;i < message->PoolElementPtrArraySize;i++) {
             if(message->PoolElementPtrArray[i]) {
                ST_CLASS(poolElementNodeDelete)(message->PoolElementPtrArray[i]);
+
                transportAddressBlockDelete(message->PoolElementPtrArray[i]->UserTransport);
                free(message->PoolElementPtrArray[i]->UserTransport);
+               message->PoolElementPtrArray[i]->UserTransport = NULL;
+
+               transportAddressBlockDelete(message->PoolElementPtrArray[i]->RegistratorTransport);
+               free(message->PoolElementPtrArray[i]->RegistratorTransport);
+               message->PoolElementPtrArray[i]->RegistratorTransport = NULL;
+
                free(message->PoolElementPtrArray[i]);
                message->PoolElementPtrArray[i] = NULL;
             }
@@ -132,7 +150,11 @@ void rserpoolMessageClearAll(struct RSerPoolMessage* message)
       }
       if((message->PeerListNodePtrAutoDelete) && (message->PeerListNodePtr)) {
          ST_CLASS(peerListNodeDelete)(message->PeerListNodePtr);
+         transportAddressBlockDelete(message->PeerListNodePtr->AddressBlock);
+         free(message->PeerListNodePtr->AddressBlock);
+         message->PeerListNodePtr->AddressBlock = NULL;
          free(message->PeerListNodePtr);
+         message->PeerListNodePtr = NULL;
       }
       if((message->PeerListPtrAutoDelete) && (message->PeerListPtr)) {
          peerListNode = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(
@@ -142,10 +164,13 @@ void rserpoolMessageClearAll(struct RSerPoolMessage* message)
             ST_CLASS(peerListNodeDelete)(peerListNode);
             transportAddressBlockDelete(peerListNode->AddressBlock);
             free(peerListNode->AddressBlock);
-            free(message->PeerListNodePtr);
+            peerListNode->AddressBlock = NULL;
+            free(peerListNode);
             peerListNode = ST_CLASS(peerListGetFirstPeerListNodeFromIndexStorage)(
                               message->PeerListPtr);
          }
+         free(message->PeerListPtr);
+         message->PeerListPtr = NULL;
       }
       if((message->HandlespacePtrAutoDelete) && (message->HandlespacePtr)) {
          ST_CLASS(poolHandlespaceManagementClear)(message->HandlespacePtr);
@@ -155,9 +180,11 @@ void rserpoolMessageClearAll(struct RSerPoolMessage* message)
       }
       if((message->OffendingParameterTLV) && (message->OffendingParameterTLVAutoDelete)) {
          free(message->OffendingParameterTLV);
+         message->OffendingParameterTLV = NULL;
       }
       if((message->OffendingMessageTLV) && (message->OffendingMessageTLVAutoDelete)) {
          free(message->OffendingMessageTLV);
+         message->OffendingMessageTLV = NULL;
       }
 
       buffer             = message->Buffer;
@@ -183,12 +210,12 @@ void rserpoolMessageClearBuffer(struct RSerPoolMessage* message)
 
 
 /* ###### Send RSerPoolMessage ########################################### */
-bool rserpoolMessageSend(int                     protocol,
-                         int                     fd,
-                         sctp_assoc_t            assocID,
-                         int                     flags,
-                         const card64            timeout,
-                         struct RSerPoolMessage* message)
+bool rserpoolMessageSend(int                      protocol,
+                         int                      fd,
+                         sctp_assoc_t             assocID,
+                         int                      flags,
+                         const unsigned long long timeout,
+                         struct RSerPoolMessage*  message)
 {
    size_t   messageLength;
    ssize_t  sent;
@@ -214,7 +241,7 @@ bool rserpoolMessageSend(int                     protocol,
          return(true);
       }
       LOG_ERROR
-      logerror("Write error");
+      logerror("sendtoplus() error");
       LOG_END
    }
    else {
