@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessage.h,v 1.12 2004/11/22 15:28:11 dreibh Exp $
+ *  $Id: rserpoolmessage.h 713 2005-08-17 15:31:56Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -51,6 +51,10 @@ extern "C" {
 #endif
 
 
+/* Set internal limit */
+#define MAX_MAX_HANDLE_RESOLUTION_ITEMS 128
+
+
 #define PORT_ASAP 3863
 #define PORT_ENRP 3864
 
@@ -58,21 +62,24 @@ extern "C" {
 #define PPID_ENRP 12 /* old value: 0xFAEEB5D2 */
 
 
-#define AHT_ASAP_MODIFIER            0xaa00
-#define AHT_REGISTRATION             (0x01 | AHT_ASAP_MODIFIER)
-#define AHT_DEREGISTRATION           (0x02 | AHT_ASAP_MODIFIER)
-#define AHT_REGISTRATION_RESPONSE    (0x03 | AHT_ASAP_MODIFIER)
-#define AHT_DEREGISTRATION_RESPONSE  (0x04 | AHT_ASAP_MODIFIER)
-#define AHT_NAME_RESOLUTION          (0x05 | AHT_ASAP_MODIFIER)
-#define AHT_NAME_RESOLUTION_RESPONSE (0x06 | AHT_ASAP_MODIFIER)
-#define AHT_ENDPOINT_KEEP_ALIVE      (0x07 | AHT_ASAP_MODIFIER)
-#define AHT_ENDPOINT_KEEP_ALIVE_ACK  (0x08 | AHT_ASAP_MODIFIER)
-#define AHT_ENDPOINT_UNREACHABLE     (0x09 | AHT_ASAP_MODIFIER)
-#define AHT_SERVER_ANNOUNCE          (0x0a | AHT_ASAP_MODIFIER)
-#define AHT_COOKIE                   (0x0b | AHT_ASAP_MODIFIER)
-#define AHT_COOKIE_ECHO              (0x0c | AHT_ASAP_MODIFIER)
-#define AHT_BUSINESS_CARD            (0x0d | AHT_ASAP_MODIFIER)
-#define AHT_PEER_ERROR               (0x0e | AHT_ASAP_MODIFIER)
+#define AHT_ASAP_MODIFIER              0xaa00
+#define AHT_REGISTRATION               (0x01 | AHT_ASAP_MODIFIER)
+#define AHT_DEREGISTRATION             (0x02 | AHT_ASAP_MODIFIER)
+#define AHT_REGISTRATION_RESPONSE      (0x03 | AHT_ASAP_MODIFIER)
+#define AHT_DEREGISTRATION_RESPONSE    (0x04 | AHT_ASAP_MODIFIER)
+#define AHT_HANDLE_RESOLUTION          (0x05 | AHT_ASAP_MODIFIER)
+#define AHT_HANDLE_RESOLUTION_RESPONSE (0x06 | AHT_ASAP_MODIFIER)
+#define AHT_ENDPOINT_KEEP_ALIVE        (0x07 | AHT_ASAP_MODIFIER)
+#define AHT_ENDPOINT_KEEP_ALIVE_ACK    (0x08 | AHT_ASAP_MODIFIER)
+#define AHT_ENDPOINT_UNREACHABLE       (0x09 | AHT_ASAP_MODIFIER)
+#define AHT_SERVER_ANNOUNCE            (0x0a | AHT_ASAP_MODIFIER)
+#define AHT_COOKIE                     (0x0b | AHT_ASAP_MODIFIER)
+#define AHT_COOKIE_ECHO                (0x0c | AHT_ASAP_MODIFIER)
+#define AHT_BUSINESS_CARD              (0x0d | AHT_ASAP_MODIFIER)
+#define AHT_ERROR                      (0x0e | AHT_ASAP_MODIFIER)
+
+
+#define AHF_ENDPOINT_KEEP_ALIVE_HOME   (1 << 0)
 
 
 struct rserpool_header
@@ -82,9 +89,6 @@ struct rserpool_header
    uint16_t ah_length;
 };
 
-#define AHF_REGISTRATION_REJECT   (1 << 0)
-#define AHF_DEREGISTRATION_REJECT (1 << 0)
-
 
 struct rserpool_tlv_header
 {
@@ -113,7 +117,6 @@ struct rserpool_tlv_header
 #define ATT_COOKIE                  0x0b
 #define ATT_POOL_ELEMENT_IDENTIFIER 0x0c
 #define ATT_POOL_ELEMENT_CHECKSUM   0x0d
-#define ATT_REGISTRAR_IDENTIFIER    0x3f
 
 
 struct rserpool_poolelementparameter
@@ -236,18 +239,17 @@ struct rserpool_errorcause
 
 
 
-#define EHT_ENRP_MODIFIER            0xee00
-#define EHT_PEER_PRESENCE            (0x01 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_NAME_TABLE_REQUEST  (0x02 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_NAME_TABLE_RESPONSE (0x03 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_NAME_UPDATE         (0x04 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_LIST_REQUEST        (0x05 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_LIST_RESPONSE       (0x06 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_INIT_TAKEOVER       (0x07 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_INIT_TAKEOVER_ACK   (0x08 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_TAKEOVER_SERVER     (0x09 | EHT_ENRP_MODIFIER)
-#define EHT_PEER_OWNERSHIP_CHANGE    (0x0a | EHT_ENRP_MODIFIER)
-#define EHT_PEER_ERROR               (0x0b | EHT_ENRP_MODIFIER)
+#define EHT_ENRP_MODIFIER         0xee00
+#define EHT_PRESENCE              (0x01 | EHT_ENRP_MODIFIER)
+#define EHT_HANDLE_TABLE_REQUEST  (0x02 | EHT_ENRP_MODIFIER)
+#define EHT_HANDLE_TABLE_RESPONSE (0x03 | EHT_ENRP_MODIFIER)
+#define EHT_HANDLE_UPDATE         (0x04 | EHT_ENRP_MODIFIER)
+#define EHT_LIST_REQUEST          (0x05 | EHT_ENRP_MODIFIER)
+#define EHT_LIST_RESPONSE         (0x06 | EHT_ENRP_MODIFIER)
+#define EHT_INIT_TAKEOVER         (0x07 | EHT_ENRP_MODIFIER)
+#define EHT_INIT_TAKEOVER_ACK     (0x08 | EHT_ENRP_MODIFIER)
+#define EHT_TAKEOVER_SERVER       (0x09 | EHT_ENRP_MODIFIER)
+#define EHT_ERROR                 (0x0a | EHT_ENRP_MODIFIER)
 
 
 struct rserpool_peerpresenceparameter
@@ -292,11 +294,11 @@ struct rserpool_targetparameter
 };
 
 
-#define EHF_PEER_PRESENCE_REPLY_REQUIRED              (1 << 0)
-#define EHF_PEER_NAME_TABLE_REQUEST_OWN_CHILDREN_ONLY (1 << 0)
-#define EHT_PEER_LIST_RESPONSE_REJECT                 (1 << 0)
-#define EHT_PEER_NAME_TABLE_RESPONSE_REJECT           (1 << 0)
-#define EHT_PEER_NAME_TABLE_RESPONSE_MORE_TO_SEND     (1 << 1)
+#define EHF_PRESENCE_REPLY_REQUIRED                (1 << 0)
+#define EHF_HANDLE_TABLE_REQUEST_OWN_CHILDREN_ONLY (1 << 0)
+#define EHF_LIST_RESPONSE_REJECT                   (1 << 0)
+#define EHF_HANDLE_TABLE_RESPONSE_REJECT           (1 << 0)
+#define EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND     (1 << 1)
 
 
 struct RSerPoolMessage
@@ -325,7 +327,7 @@ struct RSerPoolMessage
    size_t                                      Position;
 
    PoolElementIdentifierType                   Identifier;
-   PoolElementChecksumType                     Checksum;
+   HandlespaceChecksumType                     Checksum;
    struct PoolPolicySettings                   PolicySettings;
    struct PoolHandle                           Handle;
 
@@ -343,7 +345,7 @@ struct RSerPoolMessage
    struct TransportAddressBlock*               TransportAddressBlockListPtr;
    bool                                        TransportAddressBlockListPtrAutoDelete;
 
-   struct ST_CLASS(PoolElementNode)*           PoolElementPtrArray[MAX_MAX_NAME_RESOLUTION_ITEMS];
+   struct ST_CLASS(PoolElementNode)*           PoolElementPtrArray[MAX_MAX_HANDLE_RESOLUTION_ITEMS];
    size_t                                      PoolElementPtrArraySize;
    bool                                        PoolElementPtrArrayAutoDelete;
 
@@ -355,10 +357,9 @@ struct RSerPoolMessage
    struct ST_CLASS(PoolHandlespaceManagement)* HandlespacePtr;
    bool                                        HandlespacePtrAutoDelete;
 
-   struct ST_CLASS(NameTableExtract)*          ExtractContinuation;
+   struct ST_CLASS(HandleTableExtract)*        ExtractContinuation;
 
    sctp_assoc_t                                AssocID;
-   unsigned short                              StreamID;
    uint32_t                                    PPID;
    union sockaddr_union                        SourceAddress;
 };
@@ -407,12 +408,12 @@ void rserpoolMessageClearBuffer(struct RSerPoolMessage* message);
   * @param message RSerPoolMessage.
   * @return true in case of success; false otherwise.
   */
-bool rserpoolMessageSend(int                     protocol,
-                         int                     fd,
-                         sctp_assoc_t            assocID,
-                         int                     flags,
-                         const card64            timeout,
-                         struct RSerPoolMessage* message);
+bool rserpoolMessageSend(int                      protocol,
+                         int                      fd,
+                         sctp_assoc_t             assocID,
+                         int                      flags,
+                         const unsigned long long timeout,
+                         struct RSerPoolMessage*  message);
 
 /**
   * For internal usage only!
