@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessagecreator.c,v 1.21 2004/11/22 15:28:11 dreibh Exp $
+ *  $Id: rserpoolmessagecreator.c 697 2005-08-08 09:26:18Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -183,7 +183,7 @@ static bool createAddressParameter(struct RSerPoolMessage* message,
        break;
       default:
          LOG_ERROR
-         fputs("Unknown address family\n", stdlog);
+         fprintf(stdlog, "Unknown address family %u\n", address->sa_family);
          LOG_END_FATAL
          return(false);
        break;
@@ -500,46 +500,23 @@ static bool createPoolElementIdentifierParameter(
 }
 
 
-/* ###### Create NS identifier parameter ################################# */
-static bool createRegistrarIdentifierParameter(
-               struct RSerPoolMessage*       message,
-               const RegistrarIdentifierType registrarIdentifier)
-{
-   uint32_t* identifier;
-   size_t    tlvPosition;
-
-   /* ATT_REGISTRAR_IDENTIFIER is optional => Also set ATT_ACTION_CONTINUE! */
-   if(beginTLV(message, &tlvPosition, ATT_REGISTRAR_IDENTIFIER|ATT_ACTION_CONTINUE) == false) {
-      return(false);
-   }
-
-   identifier = (uint32_t*)getSpace(message, sizeof(uint32_t));
-   if(identifier == NULL) {
-      return(false);
-   }
-   *identifier = htonl(registrarIdentifier);
-
-   return(finishTLV(message, tlvPosition));
-}
-
-
 /* ###### Create pool element checksum parameter ####################### */
-static bool createPoolElementChecksumParameter(
+static bool createHandlespaceChecksumParameter(
                struct RSerPoolMessage*       message,
-               const PoolElementChecksumType poolElementChecksum)
+               const HandlespaceChecksumType poolElementChecksum)
 {
-   uint32_t* checksum;
+   uint16_t* checksum;
    size_t    tlvPosition = 0;
 
    if(beginTLV(message, &tlvPosition, ATT_POOL_ELEMENT_CHECKSUM) == false) {
       return(false);
    }
 
-   checksum = (uint32_t*)getSpace(message, sizeof(uint32_t));
+   checksum = (uint16_t*)getSpace(message, sizeof(uint16_t));
    if(checksum == NULL) {
       return(false);
    }
-   *checksum = htonl(poolElementChecksum);
+   *checksum = poolElementChecksum;
 
    return(finishTLV(message, tlvPosition));
 }
@@ -642,14 +619,20 @@ static bool createServerInformationParameter(struct RSerPoolMessage*        mess
 /* ###### Create endpoint keepalive message ############################## */
 static bool createEndpointKeepAliveMessage(struct RSerPoolMessage* message)
 {
-   if(beginMessage(message, AHT_ENDPOINT_KEEP_ALIVE, message->Flags & 0x00, PPID_ASAP) == NULL) {
+   uint32_t* identifier;
+
+   if(beginMessage(message, AHT_ENDPOINT_KEEP_ALIVE,
+                   message->Flags & AHF_ENDPOINT_KEEP_ALIVE_HOME, PPID_ASAP) == NULL) {
       return(false);
    }
 
-   if(createPoolHandleParameter(message, &message->Handle) == false) {
+   identifier = (uint32_t*)getSpace(message, sizeof(uint32_t));
+   if(identifier == NULL) {
       return(false);
    }
-   if(createPoolElementIdentifierParameter(message, message->Identifier) == false) {
+   *identifier = htonl(message->RegistrarIdentifier);
+
+   if(createPoolHandleParameter(message, &message->Handle) == false) {
       return(false);
    }
 
@@ -699,7 +682,7 @@ static bool createRegistrationMessage(struct RSerPoolMessage* message)
    CHECK(message->PoolElementPtr->RegistratorTransport == NULL);
 
    if(beginMessage(message, AHT_REGISTRATION,
-                   message->Flags & AHF_REGISTRATION_REJECT, PPID_ASAP) == NULL) {
+                   message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
 
@@ -730,7 +713,7 @@ static bool createRegistrationResponseMessage(struct RSerPoolMessage* message)
    }
 
    if(beginMessage(message, AHT_REGISTRATION_RESPONSE,
-                   message->Flags & AHF_REGISTRATION_REJECT, PPID_ASAP) == NULL) {
+                   message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
 
@@ -740,6 +723,11 @@ static bool createRegistrationResponseMessage(struct RSerPoolMessage* message)
    if(createPoolElementIdentifierParameter(message, message->Identifier) == false) {
       return(false);
    }
+   if(message->Error != 0x00) {
+      if(createErrorParameter(message) == false) {
+         return(false);
+      }
+   }
 
    return(finishMessage(message));
 }
@@ -767,7 +755,7 @@ static bool createDeregistrationMessage(struct RSerPoolMessage* message)
 static bool createDeregistrationResponseMessage(struct RSerPoolMessage* message)
 {
    if(beginMessage(message, AHT_DEREGISTRATION_RESPONSE,
-                   message->Flags & AHF_DEREGISTRATION_REJECT, PPID_ASAP) == NULL) {
+                   message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
 
@@ -787,10 +775,10 @@ static bool createDeregistrationResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Create name resolution message ################################# */
-static bool createNameResolutionMessage(struct RSerPoolMessage* message)
+/* ###### Create handle resolution message ################################# */
+static bool createHandleResolutionMessage(struct RSerPoolMessage* message)
 {
-   if(beginMessage(message, AHT_NAME_RESOLUTION, message->Flags & 0x00, PPID_ASAP) == NULL) {
+   if(beginMessage(message, AHT_HANDLE_RESOLUTION, message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
    if(createPoolHandleParameter(message, &message->Handle) == false) {
@@ -800,13 +788,13 @@ static bool createNameResolutionMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Create name resolution response message ######################## */
-static bool createNameResolutionResponseMessage(struct RSerPoolMessage* message)
+/* ###### Create handle resolution response message ######################## */
+static bool createHandleResolutionResponseMessage(struct RSerPoolMessage* message)
 {
    size_t i;
 
-   CHECK(message->PoolElementPtrArraySize < MAX_MAX_NAME_RESOLUTION_ITEMS);
-   if(beginMessage(message, AHT_NAME_RESOLUTION_RESPONSE, message->Flags & 0x00, PPID_ASAP) == NULL) {
+   CHECK(message->PoolElementPtrArraySize < MAX_MAX_HANDLE_RESOLUTION_ITEMS);
+   if(beginMessage(message, AHT_HANDLE_RESOLUTION_RESPONSE, message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
 
@@ -841,7 +829,7 @@ static bool createBusinessCardMessage(struct RSerPoolMessage* message)
    size_t i;
 
    CHECK(message->PoolElementPtrArraySize > 0);
-   CHECK(message->PoolElementPtrArraySize < MAX_MAX_NAME_RESOLUTION_ITEMS);
+   CHECK(message->PoolElementPtrArraySize < MAX_MAX_HANDLE_RESOLUTION_ITEMS);
    if(beginMessage(message, AHT_BUSINESS_CARD, message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
@@ -865,12 +853,18 @@ static bool createBusinessCardMessage(struct RSerPoolMessage* message)
 /* ###### Create server announce message ################################## */
 static bool createServerAnnounceMessage(struct RSerPoolMessage* message)
 {
+   uint32_t* identifier;
+
    if(beginMessage(message, AHT_SERVER_ANNOUNCE, message->Flags & 0x00, PPID_ASAP) == NULL) {
       return(false);
    }
-   if(createRegistrarIdentifierParameter(message, message->RegistrarIdentifier) == false) {
+
+   identifier = (uint32_t*)getSpace(message, sizeof(uint32_t));
+   if(identifier == NULL) {
       return(false);
    }
+   *identifier = htonl(message->RegistrarIdentifier);
+
    return(finishMessage(message));
 }
 
@@ -902,12 +896,12 @@ static bool createCookieEchoMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer presence ########################################### */
-static bool createPeerPresenceMessage(struct RSerPoolMessage* message)
+static bool createPresenceMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
-   if(beginMessage(message, EHT_PEER_PRESENCE,
-                   message->Flags & EHF_PEER_PRESENCE_REPLY_REQUIRED,
+   if(beginMessage(message, EHT_PRESENCE,
+                   message->Flags & EHF_PRESENCE_REPLY_REQUIRED,
                    PPID_ENRP) == NULL) {
       return(false);
    }
@@ -919,7 +913,7 @@ static bool createPeerPresenceMessage(struct RSerPoolMessage* message)
    sp->sp_sender_id   = htonl(message->SenderID);
    sp->sp_receiver_id = htonl(message->ReceiverID);
 
-   if(createPoolElementChecksumParameter(message, message->Checksum) == false) {
+   if(createHandlespaceChecksumParameter(message, message->Checksum) == false) {
       return(false);
    }
    if(createServerInformationParameter(message, message->PeerListNodePtr) == false) {
@@ -931,11 +925,11 @@ static bool createPeerPresenceMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer list request ####################################### */
-static bool createPeerListRequestMessage(struct RSerPoolMessage* message)
+static bool createListRequestMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
-   if(beginMessage(message, EHT_PEER_LIST_REQUEST,
+   if(beginMessage(message, EHT_LIST_REQUEST,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -953,7 +947,7 @@ static bool createPeerListRequestMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer list response ###################################### */
-static bool createPeerListResponseMessage(struct RSerPoolMessage* message)
+static bool createListResponseMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
    struct ST_CLASS(PeerListNode)*   peerListNode;
@@ -966,8 +960,8 @@ static bool createPeerListResponseMessage(struct RSerPoolMessage* message)
       return(false);
    }
 
-   if(beginMessage(message, EHT_PEER_LIST_RESPONSE,
-                   message->Flags & EHT_PEER_LIST_RESPONSE_REJECT,
+   if(beginMessage(message, EHT_LIST_RESPONSE,
+                   message->Flags & EHF_LIST_RESPONSE_REJECT,
                    PPID_ENRP) == NULL) {
       return(false);
    }
@@ -999,13 +993,13 @@ static bool createPeerListResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Create peer name table request ################################# */
-static bool createPeerNameTableRequestMessage(struct RSerPoolMessage* message)
+/* ###### Create peer handle table request ################################# */
+static bool createHandleTableRequestMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
-   if(beginMessage(message, EHT_PEER_NAME_TABLE_REQUEST,
-                   message->Flags & EHF_PEER_NAME_TABLE_REQUEST_OWN_CHILDREN_ONLY,
+   if(beginMessage(message, EHT_HANDLE_TABLE_REQUEST,
+                   message->Flags & EHF_HANDLE_TABLE_REQUEST_OWN_CHILDREN_ONLY,
                    PPID_ENRP) == NULL) {
       return(false);
    }
@@ -1021,20 +1015,20 @@ static bool createPeerNameTableRequestMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Create peer name table response ################################ */
-static bool createPeerNameTableResponseMessage(struct RSerPoolMessage* message)
+/* ###### Create peer handle table response ################################ */
+static bool createHandleTableResponseMessage(struct RSerPoolMessage* message)
 {
-   struct rserpool_serverparameter*   sp;
-   struct ST_CLASS(NameTableExtract)* nte;
-   struct PoolHandle*                 lastPoolHandle;
-   unsigned int                       flags;
-   int                                result;
-   size_t                             i;
-   size_t                             oldPosition;
-   struct rserpool_header*            header;
+   struct rserpool_serverparameter*     sp;
+   struct ST_CLASS(HandleTableExtract)* hte = NULL;
+   struct PoolHandle*                   lastPoolHandle;
+   unsigned int                         flags;
+   int                                  result;
+   size_t                               i;
+   size_t                               oldPosition;
+   struct rserpool_header*              header;
 
-   header = beginMessage(message, EHT_PEER_NAME_TABLE_RESPONSE,
-                         message->Flags & (EHT_PEER_NAME_TABLE_RESPONSE_REJECT|EHT_PEER_NAME_TABLE_RESPONSE_MORE_TO_SEND),
+   header = beginMessage(message, EHT_HANDLE_TABLE_RESPONSE,
+                         message->Flags & (EHF_HANDLE_TABLE_RESPONSE_REJECT|EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND),
                          PPID_ENRP);
    if(header == NULL) {
       return(false);
@@ -1048,33 +1042,33 @@ static bool createPeerNameTableResponseMessage(struct RSerPoolMessage* message)
    sp->sp_receiver_id = htonl(message->ReceiverID);
 
    if(message->PeerListNodePtr) {
-      flags = (message->Action & EHF_PEER_NAME_TABLE_REQUEST_OWN_CHILDREN_ONLY) ? NTEF_OWNCHILDSONLY : 0;
-      nte = (struct ST_CLASS(NameTableExtract)*)message->PeerListNodePtr->UserData;
-      if(nte == NULL) {
-         flags |= NTEF_START;
-         nte = (struct ST_CLASS(NameTableExtract)*)malloc(sizeof(struct ST_CLASS(NameTableExtract)));
-         if(nte == NULL) {
+      flags = (message->Action & EHF_HANDLE_TABLE_REQUEST_OWN_CHILDREN_ONLY) ? HTEF_OWNCHILDSONLY : 0;
+      hte = (struct ST_CLASS(HandleTableExtract)*)message->PeerListNodePtr->UserData;
+      if(hte == NULL) {
+         flags |= HTEF_START;
+         hte = (struct ST_CLASS(HandleTableExtract)*)malloc(sizeof(struct ST_CLASS(HandleTableExtract)));
+         if(hte == NULL) {
             return(false);
          }
-         message->PeerListNodePtr->UserData = nte;
+         message->PeerListNodePtr->UserData = hte;
       }
 
       oldPosition = message->Position;
-      result = ST_CLASS(poolHandlespaceManagementGetNameTable)(
+      result = ST_CLASS(poolHandlespaceManagementGetHandleTable)(
                   message->HandlespacePtr,
                   message->HandlespacePtr->Handlespace.HomeRegistrarIdentifier,
-                  nte,
+                  hte,
                   flags);
       if(result > 0) {
          lastPoolHandle = NULL;
-         for(i = 0;i < nte->PoolElementNodes;i++) {
-            LOG_NOTE
-            ST_CLASS(poolElementNodePrint)(nte->PoolElementNodeArray[i], stdlog, ~0);
+         for(i = 0;i < hte->PoolElementNodes;i++) {
+            LOG_VERBOSE2
+            ST_CLASS(poolElementNodePrint)(hte->PoolElementNodeArray[i], stdlog, ~0);
             fputs("\n", stdlog);
             LOG_END
 
-            if(lastPoolHandle != &nte->PoolElementNodeArray[i]->OwnerPoolNode->Handle) {
-               lastPoolHandle = &nte->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
+            if(lastPoolHandle != &hte->PoolElementNodeArray[i]->OwnerPoolNode->Handle) {
+               lastPoolHandle = &hte->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
                oldPosition = message->Position;
                if(createPoolHandleParameter(message, lastPoolHandle) == false) {
                   if(i < 1) {
@@ -1085,8 +1079,8 @@ static bool createPeerNameTableResponseMessage(struct RSerPoolMessage* message)
                }
             }
 
-            CHECK(nte->PoolElementNodeArray[i]->RegistratorTransport != NULL);
-            if(createPoolElementParameter(message, nte->PoolElementNodeArray[i]) == false) {
+            CHECK(hte->PoolElementNodeArray[i]->RegistratorTransport != NULL);
+            if(createPoolElementParameter(message, hte->PoolElementNodeArray[i]) == false) {
                if(i < 1) {
                   return(false);
                }
@@ -1095,14 +1089,19 @@ static bool createPeerNameTableResponseMessage(struct RSerPoolMessage* message)
             }
             oldPosition = message->Position;
 
-            nte->LastPoolHandle            = nte->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
-            nte->LastPoolElementIdentifier = nte->PoolElementNodeArray[i]->Identifier;
+            hte->LastPoolHandle            = hte->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
+            hte->LastPoolElementIdentifier = hte->PoolElementNodeArray[i]->Identifier;
          }
-         if((nte->PoolElementNodes == NTE_MAX_POOL_ELEMENT_NODES) ||
-            (i != nte->PoolElementNodes)) {
-            header->ah_flags |= EHT_PEER_NAME_TABLE_RESPONSE_MORE_TO_SEND;
+         if((hte->PoolElementNodes == NTE_MAX_POOL_ELEMENT_NODES) ||
+            (i != hte->PoolElementNodes)) {
+            header->ah_flags |= EHF_HANDLE_TABLE_RESPONSE_MORE_TO_SEND;
          }
       }
+
+      if(hte->PoolElementNodes < NTE_MAX_POOL_ELEMENT_NODES) {
+         free(message->PeerListNodePtr->UserData);
+         message->PeerListNodePtr->UserData = NULL;
+      }
    }
 
    return(finishMessage(message));
@@ -1110,13 +1109,13 @@ static bool createPeerNameTableResponseMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer name update ######################################## */
-static bool createPeerNameUpdateMessage(struct RSerPoolMessage* message)
+static bool createNameUpdateMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_peernameupdateparameter* pnup;
 
    CHECK(message->PoolElementPtr->RegistratorTransport != NULL);
 
-   if(beginMessage(message, EHT_PEER_NAME_UPDATE,
+   if(beginMessage(message, EHT_HANDLE_UPDATE,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -1144,11 +1143,11 @@ static bool createPeerNameUpdateMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer init takeover ###################################### */
-static bool createPeerInitTakeoverMessage(struct RSerPoolMessage* message)
+static bool createInitTakeoverMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
-   if(beginMessage(message, EHT_PEER_INIT_TAKEOVER,
+   if(beginMessage(message, EHT_INIT_TAKEOVER,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -1167,11 +1166,11 @@ static bool createPeerInitTakeoverMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer init takeover acknowledgement ###################### */
-static bool createPeerInitTakeoverAckMessage(struct RSerPoolMessage* message)
+static bool createInitTakeoverAckMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
-   if(beginMessage(message, EHT_PEER_INIT_TAKEOVER_ACK,
+   if(beginMessage(message, EHT_INIT_TAKEOVER_ACK,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -1190,11 +1189,11 @@ static bool createPeerInitTakeoverAckMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Create peer takeover server #################################### */
-static bool createPeerTakeoverServerMessage(struct RSerPoolMessage* message)
+static bool createTakeoverServerMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
-   if(beginMessage(message, EHT_PEER_TAKEOVER_SERVER,
+   if(beginMessage(message, EHT_TAKEOVER_SERVER,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -1212,88 +1211,12 @@ static bool createPeerTakeoverServerMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Create peer ownership change ################################### */
-static bool createPeerOwnershipChangeMessage(struct RSerPoolMessage* message)
-{
-   struct rserpool_serverparameter* sp;
-   struct PoolHandle*               lastPoolHandle;
-   unsigned int                     flags;
-   unsigned int                     result;
-   size_t                           oldPosition;
-   size_t                           i;
-
-   if(beginMessage(message, EHT_PEER_OWNERSHIP_CHANGE,
-                   message->Flags & 0x00,
-                   PPID_ENRP) == NULL) {
-      return(false);
-   }
-
-   sp = (struct rserpool_serverparameter*)getSpace(message, sizeof(struct rserpool_serverparameter));
-   if(sp == NULL) {
-      return(false);
-   }
-   sp->sp_sender_id   = htonl(message->SenderID);
-   sp->sp_receiver_id = htonl(message->ReceiverID);
-
-   flags = NTEF_OWNCHILDSONLY;
-   if(message->ExtractContinuation->LastPoolElementIdentifier == 0) {
-      flags |= NTEF_START;
-   }
-
-   oldPosition = message->Position;
-   result = ST_CLASS(poolHandlespaceManagementGetNameTable)(
-               message->HandlespacePtr,
-               message->RegistrarIdentifier,
-               message->ExtractContinuation,
-               flags);
-   if(result > 0) {
-      lastPoolHandle = NULL;
-      for(i = 0;i < message->ExtractContinuation->PoolElementNodes;i++) {
-         LOG_NOTE
-         ST_CLASS(poolElementNodePrint)(message->ExtractContinuation->PoolElementNodeArray[i], stdlog, ~0);
-         fputs("\n", stdlog);
-         LOG_END
-
-         if(lastPoolHandle != &message->ExtractContinuation->PoolElementNodeArray[i]->OwnerPoolNode->Handle) {
-            lastPoolHandle = &message->ExtractContinuation->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
-            oldPosition = message->Position;
-            if(createPoolHandleParameter(message, lastPoolHandle) == false) {
-               if(i < 1) {
-                  return(false);
-               }
-               message->Position = oldPosition;
-               break;
-            }
-         }
-
-         if(createPoolElementIdentifierParameter(message, message->ExtractContinuation->PoolElementNodeArray[i]->Identifier) == false) {
-            if(i < 1) {
-               return(false);
-            }
-            message->Position = oldPosition;
-            break;
-         }
-         oldPosition = message->Position;
-
-         message->ExtractContinuation->LastPoolHandle            = message->ExtractContinuation->PoolElementNodeArray[i]->OwnerPoolNode->Handle;
-         message->ExtractContinuation->LastPoolElementIdentifier = message->ExtractContinuation->PoolElementNodeArray[i]->Identifier;
-      }
-      if((message->ExtractContinuation->PoolElementNodes < NTE_MAX_POOL_ELEMENT_NODES) ||
-         (i >= message->ExtractContinuation->PoolElementNodes)) {
-         message->ExtractContinuation->LastPoolElementIdentifier = 0;
-      }
-   }
-
-   return(finishMessage(message));
-}
-
-
 /* ###### Create peer error ############################################## */
 static bool createENRPPeerErrorMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
-   if(beginMessage(message, EHT_PEER_ERROR,
+   if(beginMessage(message, EHT_ERROR,
                    message->Flags & 0x00,
                    PPID_ENRP) == NULL) {
       return(false);
@@ -1317,7 +1240,7 @@ static bool createENRPPeerErrorMessage(struct RSerPoolMessage* message)
 /* ###### Create peer error ############################################## */
 static bool createASAPPeerErrorMessage(struct RSerPoolMessage* message)
 {
-   if(beginMessage(message, AHT_PEER_ERROR,
+   if(beginMessage(message, AHT_ERROR,
                    message->Flags & 0x00,
                    PPID_ASAP) == NULL) {
       return(false);
@@ -1338,19 +1261,19 @@ size_t rserpoolMessage2Packet(struct RSerPoolMessage* message)
 
    switch(message->Type) {
       /* ====== ASAP ===================================================== */
-      case AHT_NAME_RESOLUTION:
+      case AHT_HANDLE_RESOLUTION:
           LOG_VERBOSE2
-          fputs("Creating NameResolution message...\n", stdlog);
+          fputs("Creating HandleResolution message...\n", stdlog);
           LOG_END
-          if(createNameResolutionMessage(message) == true) {
+          if(createHandleResolutionMessage(message) == true) {
              return(message->Position);
           }
        break;
-      case AHT_NAME_RESOLUTION_RESPONSE:
+      case AHT_HANDLE_RESOLUTION_RESPONSE:
           LOG_VERBOSE2
-          fputs("Creating NameResolutionResponse message...\n", stdlog);
+          fputs("Creating HandleResolutionResponse message...\n", stdlog);
           LOG_END
-          if(createNameResolutionResponseMessage(message) == true) {
+          if(createHandleResolutionResponseMessage(message) == true) {
              return(message->Position);
           }
        break;
@@ -1442,7 +1365,7 @@ size_t rserpoolMessage2Packet(struct RSerPoolMessage* message)
              return(message->Position);
           }
         break;
-       case AHT_PEER_ERROR:
+       case AHT_ERROR:
           LOG_VERBOSE2
           fputs("Creating PeerError (ASAP) message...\n", stdlog);
           LOG_END
@@ -1452,87 +1375,79 @@ size_t rserpoolMessage2Packet(struct RSerPoolMessage* message)
         break;
 
        /* ====== ENRP ==================================================== */
-       case EHT_PEER_PRESENCE:
+       case EHT_PRESENCE:
           LOG_VERBOSE2
           fputs("Creating PeerPresence message...\n", stdlog);
           LOG_END
-          if(createPeerPresenceMessage(message) == true) {
+          if(createPresenceMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_NAME_TABLE_REQUEST:
+       case EHT_HANDLE_TABLE_REQUEST:
           LOG_VERBOSE2
-          fputs("Creating PeerNameTableRequest message...\n", stdlog);
+          fputs("Creating HandleTableRequest message...\n", stdlog);
           LOG_END
-          if(createPeerNameTableRequestMessage(message) == true) {
+          if(createHandleTableRequestMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_NAME_TABLE_RESPONSE:
+       case EHT_HANDLE_TABLE_RESPONSE:
           LOG_VERBOSE2
-          fputs("Creating PeerNameTableResponse message...\n", stdlog);
+          fputs("Creating HandleTableResponse message...\n", stdlog);
           LOG_END
-          if(createPeerNameTableResponseMessage(message) == true) {
+          if(createHandleTableResponseMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_NAME_UPDATE:
+       case EHT_HANDLE_UPDATE:
           LOG_VERBOSE2
-          fputs("Creating PeerNameUpdate message...\n", stdlog);
+          fputs("Creating HandleUpdate message...\n", stdlog);
           LOG_END
-          if(createPeerNameUpdateMessage(message) == true) {
+          if(createNameUpdateMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_LIST_REQUEST:
+       case EHT_LIST_REQUEST:
           LOG_VERBOSE2
-          fputs("Creating PeerListRequest message...\n", stdlog);
+          fputs("Creating ListRequest message...\n", stdlog);
           LOG_END
-          if(createPeerListRequestMessage(message) == true) {
+          if(createListRequestMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_LIST_RESPONSE:
+       case EHT_LIST_RESPONSE:
           LOG_VERBOSE2
-          fputs("Creating PeerListResponse message...\n", stdlog);
+          fputs("Creating ListResponse message...\n", stdlog);
           LOG_END
-          if(createPeerListResponseMessage(message) == true) {
+          if(createListResponseMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_INIT_TAKEOVER:
+       case EHT_INIT_TAKEOVER:
           LOG_VERBOSE2
           fputs("Creating PeerInitTakeover message...\n", stdlog);
           LOG_END
-          if(createPeerInitTakeoverMessage(message) == true) {
+          if(createInitTakeoverMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_INIT_TAKEOVER_ACK:
+       case EHT_INIT_TAKEOVER_ACK:
           LOG_VERBOSE2
           fputs("Creating PeerInitTakeoverAck message...\n", stdlog);
           LOG_END
-          if(createPeerInitTakeoverAckMessage(message) == true) {
+          if(createInitTakeoverAckMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_TAKEOVER_SERVER:
+       case EHT_TAKEOVER_SERVER:
           LOG_VERBOSE2
           fputs("Creating PeerTakeoverServer message...\n", stdlog);
           LOG_END
-          if(createPeerTakeoverServerMessage(message) == true) {
+          if(createTakeoverServerMessage(message) == true) {
              return(message->Position);
           }
         break;
-       case EHT_PEER_OWNERSHIP_CHANGE:
-          LOG_VERBOSE2
-          fputs("Creating PeerOwnershipChange message...\n", stdlog);
-          LOG_END
-          if(createPeerOwnershipChangeMessage(message) == true) {
-             return(message->Position);
-          }
-        break;
-       case EHT_PEER_ERROR:
+       case EHT_ERROR:
           LOG_VERBOSE2
           fputs("Creating PeerError (ENRP) message...\n", stdlog);
           LOG_END
