@@ -1,5 +1,5 @@
 /*
- *  $Id: rserpoolmessageparser.c,v 1.29 2004/11/22 15:28:11 dreibh Exp $
+ *  $Id: rserpoolmessageparser.c 697 2005-08-08 09:26:18Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -45,7 +45,7 @@
 #include <ext_socket.h>
 
 
-/* ###### Scan next TLV header ############################################## */
+/* ###### Scan next TLV header ########################################### */
 static bool getNextTLV(struct RSerPoolMessage*      message,
                        size_t*                      tlvPosition,
                        struct rserpool_tlv_header** header,
@@ -93,7 +93,7 @@ static bool getNextTLV(struct RSerPoolMessage*      message,
 }
 
 
-/* ###### Handle unknown TLV ################################################ */
+/* ###### Handle unknown TLV ############################################# */
 static bool handleUnknownTLV(struct RSerPoolMessage* message,
                              const uint16_t          tlvType,
                              const size_t            tlvLength)
@@ -149,7 +149,7 @@ static bool handleUnknownTLV(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Check begin of message ############################################ */
+/* ###### Check begin of message ######################################### */
 static size_t checkBeginMessage(struct RSerPoolMessage* message,
                                 size_t*                 startPosition)
 {
@@ -193,7 +193,7 @@ static size_t checkBeginMessage(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Check finish of message ########################################### */
+/* ###### Check finish of message ######################################## */
 static bool checkFinishMessage(struct RSerPoolMessage* message,
                                const size_t            startPosition)
 {
@@ -225,7 +225,7 @@ static bool checkFinishMessage(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Check begin of TLV ################################################ */
+/* ###### Check begin of TLV ############################################# */
 static size_t checkBeginTLV(struct RSerPoolMessage* message,
                             size_t*                 tlvPosition,
                             const uint16_t          expectedType,
@@ -253,7 +253,7 @@ static size_t checkBeginTLV(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Check finish of TLV ############################################### */
+/* ###### Check finish of TLV ############################################ */
 static bool checkFinishTLV(struct RSerPoolMessage* message,
                            const size_t            tlvPosition)
 {
@@ -289,7 +289,7 @@ static bool checkFinishTLV(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Scan address parameter ############################################ */
+/* ###### Scan address parameter ######################################### */
 static bool scanAddressParameter(struct RSerPoolMessage* message,
                                  const card16            port,
                                  union sockaddr_union*   address)
@@ -376,7 +376,7 @@ static bool scanAddressParameter(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Scan user transport parameter ##################################### */
+/* ###### Scan user transport parameter ################################## */
 static bool scanTransportParameter(struct RSerPoolMessage*       message,
                                    struct TransportAddressBlock* transportAddressBlock)
 {
@@ -496,7 +496,7 @@ static bool scanTransportParameter(struct RSerPoolMessage*       message,
 }
 
 
-/* ###### Scan poolPolicySettings parameter ############################################# */
+/* ###### Scan Pool Policy parameter ##################################### */
 static bool scanPolicyParameter(struct RSerPoolMessage*    message,
                                 struct PoolPolicySettings* poolPolicySettings)
 {
@@ -815,7 +815,7 @@ static bool scanPolicyParameter(struct RSerPoolMessage*    message,
 }
 
 
-/* ###### Scan pool element paramter ######################################## */
+/* ###### Scan pool element paramter ##################################### */
 static struct ST_CLASS(PoolElementNode)* scanPoolElementParameter(
                                             struct RSerPoolMessage* message)
 {
@@ -894,7 +894,7 @@ static struct ST_CLASS(PoolElementNode)* scanPoolElementParameter(
 }
 
 
-/* ###### Scan pool handle paramter ######################################### */
+/* ###### Scan pool handle paramter ###################################### */
 static bool scanPoolHandleParameter(struct RSerPoolMessage* message,
                                     struct PoolHandle*      poolHandlePtr)
 {
@@ -934,7 +934,7 @@ static bool scanPoolHandleParameter(struct RSerPoolMessage* message,
 }
 
 
-/* ###### Scan pool element identifier parameter ############################ */
+/* ###### Scan pool element identifier parameter ######################### */
 static bool scanPoolElementIdentifierParameter(struct RSerPoolMessage* message)
 {
    uint32_t* identifier;
@@ -967,43 +967,10 @@ static bool scanPoolElementIdentifierParameter(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan NS identifier parameter ################################### */
-static bool scanRegistrarIdentifierParameter(struct RSerPoolMessage* message)
+/* ###### Scan pool element checksum parameter ########################### */
+static bool scanHandlespaceChecksumParameter(struct RSerPoolMessage* message)
 {
-   uint32_t* registrarIdentifier;
-   size_t    tlvPosition = 0;
-   size_t    tlvLength   = checkBeginTLV(message, &tlvPosition, ATT_REGISTRAR_IDENTIFIER, true);
-   if(tlvLength < sizeof(struct rserpool_tlv_header)) {
-      return(false);
-   }
-
-   tlvLength -= sizeof(struct rserpool_tlv_header);
-   if(tlvLength != sizeof(uint32_t)) {
-      LOG_WARNING
-      fputs("NS Identifier too short!\n", stdlog);
-      LOG_END
-      message->Error = RSPERR_INVALID_VALUES;
-      return(false);
-   }
-
-   registrarIdentifier = (uint32_t*)getSpace(message, tlvLength);
-   if(registrarIdentifier == NULL) {
-      return(false);
-   }
-   message->RegistrarIdentifier = ntohl(*registrarIdentifier);
-
-   LOG_VERBOSE3
-   fprintf(stdlog, "Scanned NS Identifier $%08x\n", message->Identifier);
-   LOG_END
-
-   return(checkFinishTLV(message, tlvPosition));
-}
-
-
-/* ###### Scan pool element checksum parameter ############################ */
-static bool scanPoolElementChecksumParameter(struct RSerPoolMessage* message)
-{
-   uint32_t* checksum;
+   uint16_t* checksum;
    size_t    tlvPosition = 0;
    size_t    tlvLength   = checkBeginTLV(message, &tlvPosition, ATT_POOL_ELEMENT_CHECKSUM, true);
    if(tlvLength < sizeof(struct rserpool_tlv_header)) {
@@ -1011,7 +978,7 @@ static bool scanPoolElementChecksumParameter(struct RSerPoolMessage* message)
    }
 
    tlvLength -= sizeof(struct rserpool_tlv_header);
-   if(tlvLength != sizeof(uint32_t)) {
+   if(tlvLength != sizeof(uint16_t)) {
       LOG_WARNING
       fputs("Pool element checksum too short!\n", stdlog);
       LOG_END
@@ -1019,11 +986,11 @@ static bool scanPoolElementChecksumParameter(struct RSerPoolMessage* message)
       return(false);
    }
 
-   checksum = (uint32_t*)getSpace(message, tlvLength);
+   checksum = (uint16_t*)getSpace(message, tlvLength);
    if(checksum == NULL) {
       return(false);
    }
-   message->Checksum = ntohl(*checksum);
+   message->Checksum = *checksum;
 
    LOG_VERBOSE3
    fprintf(stdlog, "Scanned pool element checksum $%08x\n", message->Checksum);
@@ -1033,7 +1000,7 @@ static bool scanPoolElementChecksumParameter(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan error parameter ############################################## */
+/* ###### Scan error parameter ########################################### */
 static bool scanErrorParameter(struct RSerPoolMessage* message)
 {
    struct rserpool_errorcause* aec;
@@ -1079,7 +1046,7 @@ static bool scanErrorParameter(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan cookie parameter ############################################# */
+/* ###### Scan cookie parameter ########################################## */
 static bool scanCookieParameter(struct RSerPoolMessage* message)
 {
    void*     cookie;
@@ -1174,7 +1141,7 @@ static struct ST_CLASS(PeerListNode)* scanServerInformationParameter(struct RSer
                              newTransportAddressBlock);
 
    LOG_VERBOSE3
-   fprintf(stdlog, "Scanned server information parameter for NS $%08x, flags=$%08x, address=",
+   fprintf(stdlog, "Scanned server information parameter for registrar $%08x, flags=$%08x, address=",
            peerListNode->Identifier, peerListNode->Flags);
    transportAddressBlockPrint(transportAddressBlock, stdlog);
    fputs("\n", stdlog);
@@ -1190,20 +1157,25 @@ static struct ST_CLASS(PeerListNode)* scanServerInformationParameter(struct RSer
 }
 
 
-/* ###### Scan endpoint keepalive message ################################### */
+/* ###### Scan endpoint keepalive message ################################ */
 static bool scanEndpointKeepAliveMessage(struct RSerPoolMessage* message)
 {
-   if(scanPoolHandleParameter(message, &message->Handle) == false) {
+   uint32_t* registrarIdentifier;
+
+   registrarIdentifier = (uint32_t*)getSpace(message, sizeof(uint32_t));
+   if(registrarIdentifier == NULL) {
       return(false);
    }
-   if(scanPoolElementIdentifierParameter(message) == false) {
+   message->RegistrarIdentifier = ntohl(*registrarIdentifier);
+
+   if(scanPoolHandleParameter(message, &message->Handle) == false) {
       return(false);
    }
    return(true);
 }
 
 
-/* ###### Scan endpoint keepalive acknowledgement message ################### */
+/* ###### Scan endpoint keepalive acknowledgement message ################ */
 static bool scanEndpointKeepAliveAckMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1216,7 +1188,7 @@ static bool scanEndpointKeepAliveAckMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan endpoint unreachable message ################################# */
+/* ###### Scan endpoint unreachable message ############################## */
 static bool scanEndpointUnreachableMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1229,7 +1201,7 @@ static bool scanEndpointUnreachableMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan registration message ######################################### */
+/* ###### Scan registration message ###################################### */
 static bool scanRegistrationMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1249,7 +1221,7 @@ static bool scanRegistrationMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan deregistration message ####################################### */
+/* ###### Scan deregistration message #################################### */
 static bool scanDeregistrationMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1262,7 +1234,7 @@ static bool scanDeregistrationMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan registration response message ################################ */
+/* ###### Scan registration response message ############################# */
 static bool scanRegistrationResponseMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1280,7 +1252,7 @@ static bool scanRegistrationResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan deregistration response message ############################## */
+/* ###### Scan deregistration response message ########################### */
 static bool scanDeregistrationResponseMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
@@ -1298,8 +1270,8 @@ static bool scanDeregistrationResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan name resolution message ###################################### */
-static bool scanNameResolutionMessage(struct RSerPoolMessage* message)
+/* ###### Scan handle resolution message ################################# */
+static bool scanHandleResolutionMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
       return(false);
@@ -1308,8 +1280,8 @@ static bool scanNameResolutionMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan name resolution response message ############################# */
-static bool scanNameResolutionResponseMessage(struct RSerPoolMessage* message)
+/* ###### Scan handle resolution response message ######################## */
+static bool scanHandleResolutionResponseMessage(struct RSerPoolMessage* message)
 {
    if(scanPoolHandleParameter(message, &message->Handle) == false) {
       return(false);
@@ -1322,9 +1294,9 @@ static bool scanNameResolutionResponseMessage(struct RSerPoolMessage* message)
 
       message->PoolElementPtrArraySize = 0;
       while(message->Position < message->BufferSize) {
-         if(message->PoolElementPtrArraySize >= MAX_MAX_NAME_RESOLUTION_ITEMS) {
+         if(message->PoolElementPtrArraySize >= MAX_MAX_HANDLE_RESOLUTION_ITEMS) {
             LOG_WARNING
-            fputs("Too many Pool Element Parameters in Name Resolution Response\n", stdlog);
+            fputs("Too many Pool Element Parameters in Handle Resolution Response\n", stdlog);
             LOG_END
             return(false);
          }
@@ -1341,12 +1313,19 @@ static bool scanNameResolutionResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan name resolution response message ############################# */
+/* ###### Scan handle resolution response message ############################# */
 static bool scanServerAnnounceMessage(struct RSerPoolMessage* message)
 {
+   uint32_t*                     registrarIdentifier;
    char                          transportAddressBlockBuffer[transportAddressBlockGetSize(1)];
    struct TransportAddressBlock* transportAddressBlock = (struct TransportAddressBlock*)&transportAddressBlockBuffer;
 
+   registrarIdentifier = (uint32_t*)getSpace(message, sizeof(uint32_t));
+   if(registrarIdentifier == NULL) {
+      return(false);
+   }
+   message->RegistrarIdentifier = ntohl(*registrarIdentifier);
+
    transportAddressBlockNew(&transportAddressBlock[0],
                             IPPROTO_SCTP,
                             getPort(&message->SourceAddress.sa),
@@ -1358,10 +1337,6 @@ static bool scanServerAnnounceMessage(struct RSerPoolMessage* message)
       return(false);
    }
 
-   if(scanRegistrarIdentifierParameter(message) == false) {
-      message->RegistrarIdentifier = 0;
-   }
-
    return(true);
 }
 
@@ -1402,7 +1377,7 @@ static bool scanBusinessCardMessage(struct RSerPoolMessage* message)
    message->PoolElementPtrArraySize = 0;
    while(message->Position < message->BufferSize) {
       while(message->Position < message->BufferSize) {
-         if(message->PoolElementPtrArraySize >= MAX_MAX_NAME_RESOLUTION_ITEMS) {
+         if(message->PoolElementPtrArraySize >= MAX_MAX_HANDLE_RESOLUTION_ITEMS) {
             LOG_WARNING
             fputs("Too many Pool Element Parameters in Business Card\n", stdlog);
             LOG_END
@@ -1426,7 +1401,7 @@ static bool scanBusinessCardMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer presence message ##################################### */
-static bool scanPeerPresenceMessage(struct RSerPoolMessage* message)
+static bool scanPresenceMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
@@ -1438,7 +1413,7 @@ static bool scanPeerPresenceMessage(struct RSerPoolMessage* message)
    message->SenderID   = ntohl(sp->sp_sender_id);
    message->ReceiverID = ntohl(sp->sp_receiver_id);
 
-   if(scanPoolElementChecksumParameter(message) == false) {
+   if(scanHandlespaceChecksumParameter(message) == false) {
       return(false);
    }
 
@@ -1452,7 +1427,7 @@ static bool scanPeerPresenceMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer list request message ################################# */
-static bool scanPeerListRequestMessage(struct RSerPoolMessage* message)
+static bool scanListRequestMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
@@ -1469,7 +1444,7 @@ static bool scanPeerListRequestMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer list response message ################################ */
-static bool scanPeerListResponseMessage(struct RSerPoolMessage* message)
+static bool scanListResponseMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
    struct ST_CLASS(PeerListNode)*   peerListNode;
@@ -1483,7 +1458,7 @@ static bool scanPeerListResponseMessage(struct RSerPoolMessage* message)
    message->SenderID   = ntohl(sp->sp_sender_id);
    message->ReceiverID = ntohl(sp->sp_receiver_id);
 
-   if(!(message->Flags & EHT_PEER_LIST_RESPONSE_REJECT)) {
+   if(!(message->Flags & EHF_LIST_RESPONSE_REJECT)) {
       while(message->Position < message->BufferSize) {
          peerListNode = scanServerInformationParameter(message);
          if(peerListNode == NULL) {
@@ -1504,12 +1479,13 @@ static bool scanPeerListResponseMessage(struct RSerPoolMessage* message)
                                            &errorCode);
          if(errorCode != RSPERR_OKAY) {
             LOG_WARNING
-            fputs("PeerListResponse contains bad peer ", stdlog);
+            fputs("ListResponse contains bad peer ", stdlog);
             ST_CLASS(peerListNodePrint)(peerListNode, stdlog, PLPO_FULL);
             fputs(". Unable to add it: ", stdlog);
             rserpoolErrorPrint(errorCode, stdlog);
             fputs("\n", stdlog);
             LOG_END
+            free(peerListNode);
             message->Error = (uint16_t)errorCode;
             return(false);
          }
@@ -1520,8 +1496,8 @@ static bool scanPeerListResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan peer name table request message ########################### */
-static bool scanPeerNameTableRequestMessage(struct RSerPoolMessage* message)
+/* ###### Scan peer handle table request message ########################### */
+static bool scanHandleTableRequestMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter* sp;
 
@@ -1537,8 +1513,8 @@ static bool scanPeerNameTableRequestMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan peer name table response message ########################## */
-static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
+/* ###### Scan peer handle table response message ########################## */
+static bool scanHandleTableResponseMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_serverparameter*  sp;
    struct ST_CLASS(PoolElementNode)* poolElementNode;
@@ -1553,7 +1529,7 @@ static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
    message->SenderID   = ntohl(sp->sp_sender_id);
    message->ReceiverID = ntohl(sp->sp_receiver_id);
 
-   if(!(message->Flags & EHT_PEER_NAME_TABLE_RESPONSE_REJECT)) {
+   if(!(message->Flags & EHF_HANDLE_TABLE_RESPONSE_REJECT)) {
       message->HandlespacePtr = (struct ST_CLASS(PoolHandlespaceManagement)*)malloc(sizeof(struct ST_CLASS(PoolHandlespaceManagement)));
       if(message->HandlespacePtr == NULL) {
          message->Error = RSPERR_OUT_OF_MEMORY;
@@ -1566,6 +1542,8 @@ static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
          poolElementNode = scanPoolElementParameter(message);
          while(poolElementNode) {
             if(poolElementNode->RegistratorTransport == NULL) {
+               free(poolElementNode->UserTransport);
+               free(poolElementNode);
                message->Error = RSPERR_INVALID_REGISTRATOR;
                return(false);
             }
@@ -1583,9 +1561,15 @@ static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
                                 -1, 0,
                                 0,
                                 &newPoolElementNode);
+
+            /* These structures were used temporarily only */
+            free(poolElementNode->UserTransport);
+            free(poolElementNode->RegistratorTransport);
+            free(poolElementNode);
+
             if(message->Error != RSPERR_OKAY) {
                LOG_WARNING
-               fputs("NameTableResponse contains bad/inconsistent entry: ", stdlog);
+               fputs("HandleTableResponse contains bad/inconsistent entry: ", stdlog);
                ST_CLASS(poolElementNodePrint)(poolElementNode, stdlog, PENPO_FULL);
                fputs(" - Unable to use it: ", stdlog);
                rserpoolErrorPrint(message->Error, stdlog);
@@ -1599,7 +1583,7 @@ static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
 
          if(scannedPoolElementParameters == 0) {
             LOG_WARNING
-            fputs("PeerNameTableResponse contains empty pool\n", stdlog);
+            fputs("HandleTableResponse contains empty pool\n", stdlog);
             LOG_END
             message->Error = RSPERR_INVALID_VALUES;
             return(false);
@@ -1611,8 +1595,8 @@ static bool scanPeerNameTableResponseMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan peer name table response message ########################## */
-static bool scanPeerNameUpdateMessage(struct RSerPoolMessage* message)
+/* ###### Scan peer handle table response message ########################## */
+static bool scanHandleUpdateMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_peernameupdateparameter* pnup;
 
@@ -1642,7 +1626,7 @@ static bool scanPeerNameUpdateMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer init takeover message ################################ */
-static bool scanPeerInitTakeoverMessage(struct RSerPoolMessage* message)
+static bool scanInitTakeoverMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
@@ -1660,7 +1644,7 @@ static bool scanPeerInitTakeoverMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer init takeover acknowledgement message ################ */
-static bool scanPeerInitTakeoverAckMessage(struct RSerPoolMessage* message)
+static bool scanInitTakeoverAckMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
@@ -1678,7 +1662,7 @@ static bool scanPeerInitTakeoverAckMessage(struct RSerPoolMessage* message)
 
 
 /* ###### Scan peer takeover server message ############################## */
-static bool scanPeerTakeoverServerMessage(struct RSerPoolMessage* message)
+static bool scanTakeoverServerMessage(struct RSerPoolMessage* message)
 {
    struct rserpool_targetparameter* tp;
 
@@ -1695,83 +1679,6 @@ static bool scanPeerTakeoverServerMessage(struct RSerPoolMessage* message)
 }
 
 
-/* ###### Scan peer take ownership message ############################### */
-static bool scanPeerOwnershipChangeMessage(struct RSerPoolMessage* message)
-{
-   struct rserpool_serverparameter*  sp;
-   struct ST_CLASS(PoolElementNode)* newPoolElementNode;
-   size_t                            scannedPoolElementIdentifiers;
-   static struct PoolPolicySettings  dummyPoolPolicySettings;
-   static char                       dummyTransportAddressBlockBuffer[transportAddressBlockGetSize(1)];
-   struct TransportAddressBlock*     dummyTransportAddressBlock = (struct TransportAddressBlock*)&dummyTransportAddressBlockBuffer;
-   struct sockaddr_in                dummyAddress;
-
-   sp = (struct rserpool_serverparameter*)getSpace(message, sizeof(struct rserpool_serverparameter));
-   if(sp == NULL) {
-      message->Error = RSPERR_INVALID_VALUES;
-      return(false);
-   }
-   message->SenderID     = ntohl(sp->sp_sender_id);
-   message->ReceiverID   = ntohl(sp->sp_receiver_id);
-
-   message->HandlespacePtr = (struct ST_CLASS(PoolHandlespaceManagement)*)malloc(sizeof(struct ST_CLASS(PoolHandlespaceManagement)));
-   if(message->HandlespacePtr == NULL) {
-      message->Error = RSPERR_OUT_OF_MEMORY;
-      return(false);
-   }
-   ST_CLASS(poolHandlespaceManagementNew)(message->HandlespacePtr, 0, NULL, NULL, NULL);
-
-
-   poolPolicySettingsNew(&dummyPoolPolicySettings);
-   dummyPoolPolicySettings.PolicyType = PPT_ROUNDROBIN;
-   memset(&dummyAddress, 0, sizeof(dummyAddress));
-   dummyAddress.sin_family = AF_INET;
-   transportAddressBlockNew(dummyTransportAddressBlock,
-                            IPPROTO_SCTP, 0xffff, 0,
-                            (union sockaddr_union*)&dummyAddress, 1);
-
-   while(scanPoolHandleParameter(message, &message->Handle)) {
-      scannedPoolElementIdentifiers = 0;
-      while(scanPoolElementIdentifierParameter(message)) {
-         scannedPoolElementIdentifiers++;
-
-         message->Error = ST_CLASS(poolHandlespaceManagementRegisterPoolElement)(
-                             message->HandlespacePtr,
-                             &message->Handle,
-                             0,
-                             message->Identifier,
-                             0,
-                             &dummyPoolPolicySettings,
-                             dummyTransportAddressBlock,
-                             dummyTransportAddressBlock,
-                             -1, 0,
-                             0,
-                             &newPoolElementNode);
-         if(message->Error != RSPERR_OKAY) {
-            LOG_WARNING
-            fprintf(stdlog,
-                    "PeerOwnershipChange contains bad/inconsistent entry for ID $%08x: ",
-                    message->Identifier);
-            rserpoolErrorPrint(message->Error, stdlog);
-            fputs("\n", stdlog);
-            LOG_END
-            return(false);
-         }
-      }
-
-      if(scannedPoolElementIdentifiers == 0) {
-         LOG_WARNING
-         fputs("PeerOwnershipChange contains empty pool\n", stdlog);
-         LOG_END
-         message->Error = RSPERR_INVALID_VALUES;
-         return(false);
-      }
-   }
-
-   return(true);
-}
-
-
 /* ###### Scan peer error message ######################################## */
 static bool scanENRPPeerErrorMessage(struct RSerPoolMessage* message)
 {
@@ -1863,19 +1770,19 @@ static bool scanMessage(struct RSerPoolMessage* message)
 
    switch(message->Type) {
       /* ====== ASAP ===================================================== */
-      case AHT_NAME_RESOLUTION:
+      case AHT_HANDLE_RESOLUTION:
          LOG_VERBOSE2
-         fputs("Scanning NameResolution message...\n", stdlog);
+         fputs("Scanning HandleResolution message...\n", stdlog);
          LOG_END
-         if(scanNameResolutionMessage(message) == false) {
+         if(scanHandleResolutionMessage(message) == false) {
             return(false);
          }
        break;
-      case AHT_NAME_RESOLUTION_RESPONSE:
+      case AHT_HANDLE_RESOLUTION_RESPONSE:
          LOG_VERBOSE2
-         fputs("Scanning NameResolutionResponse message...\n", stdlog);
+         fputs("Scanning HandleResolutionResponse message...\n", stdlog);
          LOG_END
-         if(scanNameResolutionResponseMessage(message) == false) {
+         if(scanHandleResolutionResponseMessage(message) == false) {
             return(false);
          }
        break;
@@ -1966,105 +1873,97 @@ static bool scanMessage(struct RSerPoolMessage* message)
          if(scanDeregistrationResponseMessage(message) == false) {
             return(false);
          }
-       break;
-       case AHT_PEER_ERROR:
+        break;
+      case AHT_ERROR:
           LOG_VERBOSE2
           fputs("Scanning PeerError (ASAP) message...\n", stdlog);
           LOG_END
           if(scanASAPPeerErrorMessage(message) == false) {
              return(false);
           }
-        break;
+       break;
 
        /* ====== ENRP ==================================================== */
-       case EHT_PEER_PRESENCE:
+      case EHT_PRESENCE:
           LOG_VERBOSE2
           fputs("Scanning PeerPresence message...\n", stdlog);
           LOG_END
-          if(scanPeerPresenceMessage(message) == false) {
+          if(scanPresenceMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_NAME_TABLE_REQUEST:
+       break;
+      case EHT_HANDLE_TABLE_REQUEST:
           LOG_VERBOSE2
-          fputs("Scanning PeerNameTableRequest message...\n", stdlog);
+          fputs("Scanning HandleTableRequest message...\n", stdlog);
           LOG_END
-          if(scanPeerNameTableRequestMessage(message) == false) {
+          if(scanHandleTableRequestMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_NAME_TABLE_RESPONSE:
+       break;
+      case EHT_HANDLE_TABLE_RESPONSE:
           LOG_VERBOSE2
-          fputs("Scanning PeerNameTableResponse message...\n", stdlog);
+          fputs("Scanning HandleTableResponse message...\n", stdlog);
           LOG_END
-          if(scanPeerNameTableResponseMessage(message) == false) {
+          if(scanHandleTableResponseMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_NAME_UPDATE:
+       break;
+      case EHT_HANDLE_UPDATE:
           LOG_VERBOSE2
-          fputs("Scanning PeerNameUpdate message...\n", stdlog);
+          fputs("Scanning HandleUpdate message...\n", stdlog);
           LOG_END
-          if(scanPeerNameUpdateMessage(message) == false) {
+          if(scanHandleUpdateMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_LIST_REQUEST:
+       break;
+      case EHT_LIST_REQUEST:
           LOG_VERBOSE2
-          fputs("Scanning PeerListRequest message...\n", stdlog);
+          fputs("Scanning ListRequest message...\n", stdlog);
           LOG_END
-          if(scanPeerListRequestMessage(message) == false) {
+          if(scanListRequestMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_LIST_RESPONSE:
+       break;
+      case EHT_LIST_RESPONSE:
           LOG_VERBOSE2
-          fputs("Scanning PeerListResponse message...\n", stdlog);
+          fputs("Scanning ListResponse message...\n", stdlog);
           LOG_END
-          if(scanPeerListResponseMessage(message) == false) {
+          if(scanListResponseMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_INIT_TAKEOVER:
+       break;
+      case EHT_INIT_TAKEOVER:
           LOG_VERBOSE2
           fputs("Scanning PeerInitTakeover message...\n", stdlog);
           LOG_END
-          if(scanPeerInitTakeoverMessage(message) == false) {
+          if(scanInitTakeoverMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_INIT_TAKEOVER_ACK:
+       break;
+      case EHT_INIT_TAKEOVER_ACK:
           LOG_VERBOSE2
           fputs("Scanning PeerInitTakeoverAck message...\n", stdlog);
           LOG_END
-          if(scanPeerInitTakeoverAckMessage(message) == false) {
+          if(scanInitTakeoverAckMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_TAKEOVER_SERVER:
+       break;
+      case EHT_TAKEOVER_SERVER:
           LOG_VERBOSE2
           fputs("Scanning PeerTakeoverServer message...\n", stdlog);
           LOG_END
-          if(scanPeerTakeoverServerMessage(message) == false) {
+          if(scanTakeoverServerMessage(message) == false) {
              return(false);
           }
-        break;
-       case EHT_PEER_OWNERSHIP_CHANGE:
-          LOG_VERBOSE2
-          fputs("Scanning PeerOwnershipChange message...\n", stdlog);
-          LOG_END
-          if(scanPeerOwnershipChangeMessage(message) == false) {
-             return(false);
-          }
-        break;
-       case EHT_PEER_ERROR:
+       break;
+      case EHT_ERROR:
           LOG_VERBOSE2
           fputs("Scanning PeerError (ENRP) message...\n", stdlog);
           LOG_END
           if(scanENRPPeerErrorMessage(message) == false) {
              return(false);
           }
-        break;
+       break;
 
        /* ====== Unknown ================================================= */
       default:
@@ -2082,6 +1981,7 @@ static bool scanMessage(struct RSerPoolMessage* message)
 /* ###### Convert packet to RSerPoolMessage ############################## */
 unsigned int rserpoolPacket2Message(char*                       packet,
                                     const union sockaddr_union* sourceAddress,
+                                    const sctp_assoc_t          assocID,
                                     const uint32_t              ppid,
                                     const size_t                packetSize,
                                     const size_t                minBufferSize,
@@ -2095,6 +1995,7 @@ unsigned int rserpoolPacket2Message(char*                       packet,
       else {
          memset(&(*message)->SourceAddress, 0, sizeof(*sourceAddress));
       }
+      (*message)->AssocID                                = assocID;
       (*message)->PPID                                   = ppid;
       (*message)->OriginalBufferSize                     = max(packetSize, minBufferSize);
       (*message)->Position                               = 0;
@@ -2102,7 +2003,7 @@ unsigned int rserpoolPacket2Message(char*                       packet,
       (*message)->CookiePtrAutoDelete                    = true;
       (*message)->PoolElementPtrArrayAutoDelete          = true;
       (*message)->TransportAddressBlockListPtrAutoDelete = true;
-      (*message)->HandlespacePtrAutoDelete                 = true;
+      (*message)->HandlespacePtrAutoDelete               = true;
       (*message)->PeerListNodePtrAutoDelete              = true;
       (*message)->PeerListPtrAutoDelete                  = true;
 
