@@ -1,5 +1,5 @@
 /*
- *  $Id: rsplib.c,v 1.23 2005/03/02 13:34:16 dreibh Exp $
+ *  $Id: rsplib.c 712 2005-08-17 15:05:32Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -130,6 +130,7 @@ const char* rspGetErrorDescription(const unsigned int errorCode)
 unsigned int rspAddStaticRegistrar(const char* addressString)
 {
    union sockaddr_union addressArray[MAX_PE_TRANSPORTADDRESSES];
+   unsigned int         result;
    char                 str[1024];
    size_t               addresses;
    char*                address;
@@ -160,10 +161,12 @@ unsigned int rspAddStaticRegistrar(const char* addressString)
          return(RSPERR_INVALID_VALUES);
       }
 
-      return(serverTableAddStaticEntry(
-                gAsapInstance->RegistrarTable,
-                (union sockaddr_union*)&addressArray,
-                addresses));
+      lock(&gDispatcher, NULL);
+      result = registrarTableAddStaticEntry(gAsapInstance->RegistrarSet,
+                                            (union sockaddr_union*)&addressArray,
+                                            addresses);
+      unlock(&gDispatcher, NULL);
+      return(result);
    }
    else {
       LOG_ERROR
@@ -265,11 +268,11 @@ unsigned int rspDeregister(const unsigned char* poolHandle,
 }
 
 
-/* ###### Name resolution ################################################ */
-unsigned int rspNameResolution(const unsigned char*         poolHandle,
-                               const size_t                 poolHandleSize,
-                               struct EndpointAddressInfo** endpointAddressInfo,
-                               struct TagItem*              tags)
+/* ###### Handle resolution ############################################## */
+unsigned int rspHandleResolution(const unsigned char*         poolHandle,
+                                 const size_t                 poolHandleSize,
+                                 struct EndpointAddressInfo** endpointAddressInfo,
+                                 struct TagItem*              tags)
 {
    struct PoolHandle                 myPoolHandle;
    struct ST_CLASS(PoolElementNode)* poolElementNode;
@@ -282,7 +285,7 @@ unsigned int rspNameResolution(const unsigned char*         poolHandle,
       poolHandleNew(&myPoolHandle, poolHandle, poolHandleSize);
 
       poolElementNodes = 1;
-      result = asapInstanceNameResolution(
+      result = asapInstanceHandleResolution(
                   gAsapInstance,
                   &myPoolHandle,
                   (struct ST_CLASS(PoolElementNode)**)&poolElementNode,
@@ -313,6 +316,8 @@ unsigned int rspNameResolution(const unsigned char*         poolHandle,
                for(i = 0;i < poolElementNode->UserTransport->Addresses;i++) {
                   memcpy((void*)ptr, (void*)&poolElementNode->UserTransport->AddressArray[i],
                          sizeof(union sockaddr_union));
+                  if (poolElementNode->UserTransport->AddressArray[i].sa.sa_family == AF_INET6)
+                    (*endpointAddressInfo)->ai_family = AF_INET6;
                   ptr = (char*)((long)ptr + (long)sizeof(union sockaddr_union));
                }
             }
@@ -387,18 +392,18 @@ int rspSelect(int             n,
               fd_set*         exceptfds,
               struct timeval* timeout)
 {
-   struct timeval mytimeout;
-   fd_set         myreadfds;
-   fd_set         mywritefds;
-   fd_set         myexceptfds;
-   fd_set         testfds;
-   card64         testTS;
-   int            myn;
-   int            i;
-   card64         asapTimeout;
-   card64         userTimeout;
-   card64         newTimeout;
-   int            result;
+   struct timeval     mytimeout;
+   fd_set             myreadfds;
+   fd_set             mywritefds;
+   fd_set             myexceptfds;
+   fd_set             testfds;
+   unsigned long long testTS;
+   unsigned long long asapTimeout;
+   unsigned long long userTimeout;
+   unsigned long long newTimeout;
+   int                result;
+   int                myn;
+   int                i;
 
    /* ====== Schedule ==================================================== */
    /* pthreads seem to have the property that scheduling is quite
@@ -410,15 +415,15 @@ int rspSelect(int             n,
    /* ====== Collect data for ext_select() call ========================== */
    lock(&gDispatcher, NULL);
    if(timeout == NULL) {
-      userTimeout = (card64)~0;
+      userTimeout = (unsigned long long)~0;
       mytimeout.tv_sec  = ~0;
       mytimeout.tv_usec = 0;
    }
    else {
-      userTimeout = ((card64)timeout->tv_sec * 1000000) + (card64)timeout->tv_usec;
+      userTimeout = ((unsigned long long)timeout->tv_sec * 1000000) + (unsigned long long)timeout->tv_usec;
    }
    dispatcherGetSelectParameters(&gDispatcher, &myn, &myreadfds, &mywritefds, &myexceptfds, &testfds, &testTS, &mytimeout);
-   asapTimeout = ((card64)mytimeout.tv_sec * 1000000) + (card64)mytimeout.tv_usec;
+   asapTimeout = ((unsigned long long)mytimeout.tv_sec * 1000000) + (unsigned long long)mytimeout.tv_usec;
    newTimeout  = min(userTimeout, asapTimeout);
    mytimeout.tv_sec  = newTimeout / 1000000;
    mytimeout.tv_usec = newTimeout % 1000000;
@@ -482,31 +487,39 @@ int rspSelect(int             n,
    dispatcherHandleSelectResult(&gDispatcher, result, &myreadfds, &mywritefds, &myexceptfds, &testfds, testTS);
 
    /* ====== Prepare results for user ==================================== */
-   result = 0;
    if(readfds) {
       FD_ZERO(readfds);
-      for(i = 0;i < n;i++) {
-         if(FD_ISSET(i,&myreadfds)) {
-            FD_SET(i,readfds);
-            result++;
-         }
-      }
    }
    if(writefds) {
       FD_ZERO(writefds);
-      for(i = 0;i < n;i++) {
-         if(FD_ISSET(i,&mywritefds)) {
-            FD_SET(i,writefds);
-            result++;
-         }
-      }
    }
    if(exceptfds) {
       FD_ZERO(exceptfds);
-      for(i = 0;i < n;i++) {
-         if(FD_ISSET(i,&myexceptfds)) {
-            FD_SET(i,exceptfds);
-            result++;
+   }
+   if(result > 0) {
+      result = 0;
+      if(readfds) {
+         for(i = 0;i < n;i++) {
+            if(FD_ISSET(i,&myreadfds)) {
+               FD_SET(i,readfds);
+               result++;
+            }
+         }
+      }
+      if(writefds) {
+         for(i = 0;i < n;i++) {
+            if(FD_ISSET(i,&mywritefds)) {
+               FD_SET(i,writefds);
+               result++;
+            }
+         }
+      }
+      if(exceptfds) {
+         for(i = 0;i < n;i++) {
+            if(FD_ISSET(i,&myexceptfds)) {
+               FD_SET(i,exceptfds);
+               result++;
+            }
          }
       }
    }
@@ -525,7 +538,7 @@ size_t rspGetComponentStatus(struct ComponentAssociationEntry** caeArray,
                                           statusText,
                                           componentAddress,
                                           gAsapInstance->RegistrarSocket,
-                                          gAsapInstance->RegistrarID,
-                                          gAsapInstance->RegistrarSocketProtocol,
+                                          gAsapInstance->RegistrarIdentifier,
+                                          IPPROTO_SCTP,
                                           gAsapInstance->RegistrarConnectionTimeStamp));
 }
