@@ -1,5 +1,5 @@
 /*
- *  $Id: rspsession.c,v 1.31 2005/03/02 13:34:16 dreibh Exp $
+ *  $Id: rspsession.c 754 2005-09-15 09:20:16Z dreibh $
  *
  * RSerPool implementation.
  *
@@ -104,7 +104,7 @@ struct SessionDescriptor
 
    unsigned long long            ConnectionTimeStamp;
    unsigned long long            ConnectTimeout;
-   unsigned long long            NameResolutionRetryDelay;
+   unsigned long long            HandleResolutionRetryDelay;
 
    struct MessageBuffer*         MessageBuffer;
    struct TagItem*               Tags;
@@ -283,10 +283,11 @@ static bool rspPoolElementUpdateRegistration(struct PoolElementDescriptor* ped)
       LOG_ACTION
       fprintf(stdlog, "(Re-)Registration successful, ID is $%08x\n", ped->Identifier);
       LOG_END
-printf("(Re-)Registration successful, ID is $%08x\n", ped->Identifier);
+
+      printf("(Re-)Registration successful, ID is $%08x\n", ped->Identifier);
    }
    else {
-      LOG_WARNING
+      LOG_ERROR
       fprintf(stdlog, "(Re-)Registration failed: ");
       rserpoolErrorPrint(result, stdlog);
       fputs("\n", stdlog);
@@ -337,16 +338,24 @@ static void reregistrationTimer(struct Dispatcher* dispatcher,
 void rspDeletePoolElement(struct PoolElementDescriptor* ped,
                           struct TagItem*               tags)
 {
-   GList* list;
+   GList*       list;
+   unsigned int result;
 
    if(ped) {
       list = g_list_first(ped->SessionList);
       if(list == NULL) {
          timerDelete(&ped->ReregistrationTimer);
          if(ped->Identifier != 0x00000000) {
-            rspDeregister((unsigned char*)&ped->Handle.Handle,
-                          ped->Handle.Size,
-                          ped->Identifier, tags);
+            result = rspDeregister((unsigned char*)&ped->Handle.Handle,
+                                   ped->Handle.Size,
+                                   ped->Identifier, tags);
+            if(result != RSPERR_OKAY) {
+               LOG_ERROR
+               fprintf(stdlog, "Deregistration failed: ");
+               rserpoolErrorPrint(result, stdlog);
+               fputs("\n", stdlog);
+               LOG_END
+            }
          }
          if(ped->Socket >= 0) {
             ext_close(ped->Socket);
@@ -448,7 +457,7 @@ struct PoolElementDescriptor* rspCreatePoolElement(const unsigned char* poolHand
          return(NULL);
       }
       if((ped->SocketType == SOCK_STREAM) && (ext_listen(ped->Socket, 5) < 0)) {
-         LOG_WARNING
+         LOG_ERROR
          logerror("Unable to set socket for new pool element to listen mode");
          LOG_END
       }
@@ -492,7 +501,7 @@ static struct SessionDescriptor* rspSessionNew(
          free(session);
          return(NULL);
       }
-      session->Tags = tagListDuplicate(tags);  /* ??? FILTER !!! */
+      session->Tags = tagListDuplicate(tags);
       if(session->Tags == NULL) {
          messageBufferDelete(session->MessageBuffer);
          free(session);
@@ -523,7 +532,7 @@ static struct SessionDescriptor* rspSessionNew(
       session->StatusTextText[0]         = 0x00;
       session->ConnectionTimeStamp      = 0;
       session->ConnectTimeout           = (unsigned long long)tagListGetData(tags, TAG_RspSession_ConnectTimeout, 5000000);
-      session->NameResolutionRetryDelay = (unsigned long long)tagListGetData(tags, TAG_RspSession_NameResolutionRetryDelay, 250000);
+      session->HandleResolutionRetryDelay = (unsigned long long)tagListGetData(tags, TAG_RspSession_HandleResolutionRetryDelay, 250000);
       if(session->PoolElement != NULL) {
          threadSafetyLock(&session->PoolElement->Mutex);
          session->PoolElement->SessionList =
@@ -598,7 +607,8 @@ bool rspSessionSendCookie(struct SessionDescriptor* session,
       LOG_END
       result = rserpoolMessageSend(session->SocketProtocol,
                                    session->Socket,
-                                   0, 0,
+                                   0,
+                                   (unsigned int)tagListGetData(tags, TAG_RspIO_Flags, MSG_NOSIGNAL),
                                    (unsigned long long)tagListGetData(tags, TAG_RspIO_Timeout, (tagdata_t)~0),
                                    message);
       threadSafetyUnlock(&session->Mutex);
@@ -626,7 +636,8 @@ static bool rspSessionSendCookieEcho(struct SessionDescriptor* session)
          LOG_END
          result = rserpoolMessageSend(session->SocketProtocol,
                                       session->Socket,
-                                      0, 0,
+                                      0,
+                                      MSG_NOSIGNAL,
                                       0,
                                       message);
          threadSafetyUnlock(&session->Mutex);
@@ -637,7 +648,7 @@ static bool rspSessionSendCookieEcho(struct SessionDescriptor* session)
 }
 
 
-/* ###### Send business card ################################################# */
+/* ###### Send business card ############################################# */
 static void rspSessionSendBusinessCard(struct SessionDescriptor* session)
 {
    if(session->PoolElement) {
@@ -650,8 +661,15 @@ static void rspSessionSendBusinessCard(struct SessionDescriptor* session)
 }
 
 
-/* ###### Do session failover ################################################*/
-static bool rspSessionFailover(struct SessionDescriptor* session)
+/* ###### Check, if session has cookie ################################### */
+bool rspSessionHasCookie(struct SessionDescriptor* session)
+{
+   return(session->Cookie != NULL);
+}
+
+
+/* ###### Do session failover ############################################ */
+bool rspSessionFailover(struct SessionDescriptor* session)
 {
    struct EndpointAddressInfo* eai;
    struct EndpointAddressInfo* eai2;
@@ -662,7 +680,7 @@ static bool rspSessionFailover(struct SessionDescriptor* session)
    threadSafetyLock(&session->Mutex);
 
    if(session->Socket >= 0) {
-      /* ====== Close connection ============================================ */
+      /* ====== Close connection ========================================= */
       LOG_ACTION
       fprintf(stdlog, "Closing socket %d\n", session->Socket);
       LOG_END
@@ -705,17 +723,17 @@ static bool rspSessionFailover(struct SessionDescriptor* session)
       }
    }
 
-   /* ====== Do name resolution ============================================= */
+   /* ====== Do handle resolution ======================================== */
    if(session->Handle.Size > 0) {
       LOG_ACTION
-      fputs("Doing name resolution\n", stdlog);
+      fputs("Doing handle resolution\n", stdlog);
       LOG_END
-      result = rspNameResolution((unsigned char*)&session->Handle.Handle,
+      result = rspHandleResolution((unsigned char*)&session->Handle.Handle,
                                  session->Handle.Size,
                                  &eai, NULL);
       if(result == RSPERR_OKAY) {
 
-         /* ====== Establish connection ======================================== */
+         /* ====== Establish connection ================================== */
          eai2 = eai;
          while(eai2 != NULL) {
             LOG_VERBOSE
@@ -765,7 +783,7 @@ static bool rspSessionFailover(struct SessionDescriptor* session)
             eai2 = eai2->ai_next;
          }
 
-         /* ====== Free name resolution result ================================= */
+         /* ====== Free handle resolution result ========================= */
          rspFreeEndpointAddressArray(eai);
 
          if(session->Socket >= 0) {
@@ -785,14 +803,14 @@ static bool rspSessionFailover(struct SessionDescriptor* session)
       else if(result == RSPERR_NOT_FOUND) {
          LOG_ACTION
          fprintf(stdlog,
-                 "Name resolution did not find new pool element. Waiting %lluus...\n",
-                 session->NameResolutionRetryDelay);
+                 "Handle resolution did not find new pool element. Waiting %lluus...\n",
+                 session->HandleResolutionRetryDelay);
          LOG_END
-         usleep((unsigned int)session->NameResolutionRetryDelay);
+         usleep((unsigned int)session->HandleResolutionRetryDelay);
       }
       else {
          LOG_WARNING
-         fputs("Name resolution not successful: ", stdlog);
+         fputs("Handle resolution not successful: ", stdlog);
          rserpoolErrorPrint(result, stdlog);
          fputs("\n", stdlog);
          LOG_END
@@ -894,7 +912,7 @@ static unsigned int handleRSerPoolMessage(struct SessionDescriptor* session,
 
    threadSafetyLock(&session->Mutex);
 
-   result = rserpoolPacket2Message(buffer, NULL, PPID_ASAP, size, size, &message);
+   result = rserpoolPacket2Message(buffer, NULL, 0, PPID_ASAP, size, size, &message);
    if(message != NULL) {
       if(result == RSPERR_OKAY) {
          LOG_VERBOSE2
@@ -1042,7 +1060,8 @@ ssize_t rspSessionRead(struct SessionDescriptor* session,
          }
          return(RspRead_ControlRead);
       }
-      else if(result == RspRead_ReadError) {
+      else if((result == 0) ||
+              (result == RspRead_ReadError)) {
          if(tagListGetData(tags, TAG_RspIO_MakeFailover, 1) != 0) {
             LOG_VERBOSE
             fprintf(stdlog, "Session failure during read, socket %d. Failover necessary\n",
@@ -1115,17 +1134,18 @@ ssize_t rspSessionWrite(struct SessionDescriptor* session,
    ssize_t result = sendtoplus(session->Socket, buffer, length,
                                tagListGetData(tags, TAG_RspIO_Flags, MSG_NOSIGNAL),
                                NULL, 0,
+                               tagListGetData(tags, TAG_RspIO_SCTP_PPID, 0x55555555),
                                0,
-                               (const sctp_assoc_t)tagListGetData(tags, TAG_RspIO_SCTP_StreamID, 0),
-                               tagListGetData(tags, TAG_RspIO_SCTP_PPID,       0),
+                               (const uint16_t)tagListGetData(tags, TAG_RspIO_SCTP_StreamID, 0),
                                tagListGetData(tags, TAG_RspIO_SCTP_TimeToLive, 0xffffffff),
-                               tagListGetData(tags, TAG_RspIO_Timeout,         (tagdata_t)~0));
+                               tagListGetData(tags, TAG_RspIO_Timeout, (tagdata_t)~0));
    if((result < 0) && (errno != EAGAIN)) {
       LOG_ACTION
       fprintf(stdlog, "Session failure during write, socket %d. Failover necessary\n",
               session->Socket);
       LOG_END
       rspSessionFailover(session);
+      return(RspRead_Failover);
    }
    tagListSetData(tags, TAG_RspIO_PE_ID, session->Identifier);
    return(result);
@@ -1344,6 +1364,7 @@ size_t rspSessionCreateComponentStatus(
                (*caeArray)[caeArraySize].ProtocolID = session->SocketProtocol;
                (*caeArray)[caeArraySize].PPID       = 0;
                caeArraySize++;
+               componentStatusGetComponentAddress(componentAddress, session->Socket, 0);
             }
             if(session->StatusTextText[0] != 0x00) {
                safestrcpy(statusText,
